<?php

namespace App\Controllers;
use Illuminate\Database\Capsule\Manager as DB;
use Abonos ;
use Contadores ;
use Tickets ;
use Albaranes ;
use DateTime ;
use LineasVenta ;
use Slim\Views\Twig;
use Slim\Http\Request;
use Slim\Http\Response ;


final class AbonosController
{
    private $app;
    private $view;
    private $utiles;
    private $pdo;

    public function __construct($app, Twig $view)
    {
        $this->app = $app;
        $this->view = $view;
        $container = $this->app->getContainer();
        $this->utiles = $container->get('Utiles');
        $this->pdo = $container->get('Pdo');
    }


    public function index()
    {
        echo ':)';
    }

    public function lista(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $data = DB::table('abonos as t')
            ->where('t.ultimo',1)
            ->leftJoin('clientes as cl', 'cl.id', '=', 't.cliente_id')
            ->leftJoin('tarifas_golf as tg', 'tg.id', '=', 't.tarifa_id')
            ->select('t.*','cl.razon_social','tg.descripcion as tarifagolf_descripcion')
            ->orderBy('t.abono')->get();

        $array = array('resultado' => 'ok','datos' => $data);

        return $response->withJson($array);
    }

    public function buscar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $busqueda=$datos['parametro'];

        $data = DB::table('abonos as t')
            ->leftJoin('clientes as cl', 'cl.id', '=', 't.cliente_id')
            ->leftJoin('productos', 'productos.id', '=', 't.producto_id')
            ->leftJoin('tarifas_golf as tg', 'tg.id', '=', 't.tarifa_id')
            ->select('t.*','cl.razon_social','productos.descripcion as producto_descripcion',
                    'tg.descripcion as tarifa_descripcion','tg.tarifa as tarifa_codigo')
            ->orderBy('t.abono');

        $data->whereRaw('t.ultimo = 1 and (cl.razon_social like "%'. strtoupper($busqueda) . '%" or t.nombre_jugador like "%'.strtoupper($busqueda).'%" or t.abono = "'.$busqueda.'")');
        $abonos = $data->get();

        $array = array('resultado' => 'ok','datos' => $abonos);

        return $response->withJson($array);
    }

    public function buscarAbonosSocios(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $busqueda=$datos['parametro'];

        $data = DB::table('abonos as t')
            ->leftJoin('clientes as cl', 'cl.id', '=', 't.cliente_id')
            ->leftJoin('productos', 'productos.id', '=', 't.producto_id')
            ->leftJoin('tarifas_golf as tg', 'tg.id', '=', 't.tarifa_id')
            ->select('t.*','cl.razon_social','productos.descripcion as producto_descripcion',
                'tg.descripcion as tarifa_descripcion','tg.tarifa as tarifa_codigo')
            ->orderBy('t.abono');
        $data->whereRaw('t.ultimo = 1 and (cl.razon_social like "%'. strtoupper($busqueda) . '%" or t.nombre_jugador like "%'.strtoupper($busqueda).'%" or t.abono = "'.$busqueda.'")');
        $abonos = $data->get();

        $data = DB::table('clientes as t')
            ->select('t.*','t.razon_social')
            ->orderBy('t.razon_social');
        $data->whereRaw('t.razon_social like "%'. strtoupper($busqueda).'%" ');
        $clientes = $data->get();

        $datos = [];
        foreach( $abonos as $abono) {
            $datos[] = ['tipo'=>'A','datos'=>$abono] ;
        }
        foreach( $clientes as $cliente) {
            $datos[] = ['tipo'=>'C','datos'=>$cliente] ;
        }
        $array = array('resultado' => 'ok','datos' => $datos);

        return $response->withJson($array);
    }

    public function abonosSinFacturar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $cliente_id=$datos['cliente_id'];

        $data = DB::table('abonos as t')
            ->where('t.ultimo',1)
            ->where('t.cliente_id',$cliente_id)
            ->where('t.albaran_id',0)
            ->where('t.ticket_id',0)
            ->leftJoin('clientes as cl', 'cl.id', '=', 't.cliente_id')
            ->leftJoin('productos', 'productos.id', '=', 't.producto_id')
            ->leftJoin('tarifas_golf as tg', 'tg.id', '=', 't.tarifa_id')
            ->select('t.*','cl.razon_social','productos.descripcion as producto_descripcion',
                'tg.descripcion as tarifa_descripcion','tg.tarifa as tarifa_codigo')
            ->orderBy('t.abono');

        $abonos = $data->get();

        $array = array('resultado' => 'ok','datos' => $abonos);

        return $response->withJson($array);
    }

    public function verRenovaciones(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $abonado = $datos['abono'];
        $data = DB::table('abonos as t')
            ->where('t.abono',$abonado)
            ->leftJoin('clientes', 'clientes.id', '=', 't.cliente_id')
            ->leftJoin('productos as pr', 'pr.id', '=', 't.producto_id')
            ->select('t.*','clientes.razon_social','pr.descripcion as descripcion_producto')
            ->orderBy('t.fechaini','desc')->get();

        $array = array('resultado' => 'ok','datos' => $data);

        return $response->withJson($array);
    }

    public function ver(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $id = $datos['id'];
        $abono = DB::table('abonos as t')->where('t.id',$id)->first();
        $array = array('resultado' => 'ok','datos' => $abono,'id'=>$id,'fecha_hoy'=> date("Y-m-d"));
        return $response->withJson($array);
    }

    public function verDisponibilidad(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $id = $datos['id'];
        $abonado = $datos['abono'];
        $datosAbonado = $this->actualizarAbono($abonado);
        $array = array('resultado' => 'ok','datos' => $datosAbonado,'id'=>$id,'fecha_hoy'=> date("Y-m-d"));
        return $response->withJson($array);
    }

    public function grabar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $id = $datos['id'];
        $abonado = $datos['abono'];

        // si es un registro nuevo comprobamos si el número de abonado existe en la bbbdd.
        if ($id == 0) {
            $abono = DB::table('abonos')->where('abono', '=', $datos['abono'])->get();
            if ($abono) {
                $mensajes['cliente_id'][]= ['error'=>'required','messages'=> 'Este número de abonado ya existe'];
                $respuesta = array('resultado' => 'ko','mensajes'=>json_encode($mensajes));
                echo json_encode($respuesta,JSON_NUMERIC_CHECK);
                return ;
            }
        }

        if ($id == 999999) {
            $datos['id'] = 0 ;
        }

        $registro = $this->utiles->grabaModelo('Abonos',$datos);

        $this->actualizarAbono($abonado);

        $resultado = $registro['resultado'];
        $mensajes =  $registro['mensajes'];
        if ($resultado == 'ok') {
            $respuesta = array('resultado' => 'ok','id'=>$registro['id']);
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }


    public function actualizar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $abonado = $datos['abono'];
        $abono = $this->actualizarAbono($abonado);
        $respuesta = array('resultado' => 'ok','abono'=>$abono,'fecha_hoy'=> date("Y-m-d"));
        return $response->withJson($respuesta);
    }


    public function renovacion(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $id = $datos['id'];

        $abono = DB::table('abonos')->where('id', '=', $id)->first();

        $abono->fechaini = date("Y-m-d");
        $abono->fechafin = date('Y-m-d',strtotime(date("Y-m-d", time()) . " + 365 day"));

        $respuesta = array('resultado' => 'ok','datos'=>$abono,'fecha_hoy'=> date("Y-m-d"));
        return $response->withJson($respuesta);
    }

    public function generar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $datosAbono = $datos['abono'];
        $numeroAbonados = $datos['numero_abonados'];

        $abonosGenerados = [];

        for($i=1; $i<=$numeroAbonados;$i++) {

            $abono = $this->numeroAbono() ;
            $datosAbono['abono'] = $abono ;
            $datosAbono['id'] = 0 ;
            $nuevoAbono = $this->utiles->grabaModelo('Abonos',$datosAbono);

            $this->actualizarAbono($abono);
        }



        $respuesta = array('resultado' => 'ok','abonos_generados'=>$abonosGenerados);
        return $response->withJson($respuesta);
    }

    public function actualizarAbono($abonado) {
        // si es un registro nuevo comprobamos si el número de abonado existe en la bbbdd.
        $renovacionesAbono = DB::table('abonos')->where('abono', '=', $abonado)->get();

        foreach ($renovacionesAbono as $abono) {
            $totalSalidasAbono = DB::table('lineas_venta as l')
                ->leftJoin('tickets as t', 'l.ticket_id', '=', 't.id')
                ->select('l.*', 't.fecha')
                ->where('l.abono_id', '=', $abono->id)
                ->sum('l.cantidad');



            $salidas = !empty($totalSalidasAbono) ? $totalSalidasAbono : 0;
            $abonoActivo = 1;
            if ($salidas >= $abono->gf18 && $abono->anual == 0) {
                $abonoActivo = 0;
            }
            $fechaHoy = date("Y-m-d");
            if ($abono->anual == 1 && ($fechaHoy < $abono->fechaini || $fechaHoy > $abono->fechafin )) {
                $abonoActivo = 0;
            }
            DB::table('abonos')->where('id', '=', $abono->id)->update(['activo' => $abonoActivo,'gf18c'=>$salidas]);
        }

        // ponemos como ultimo los abonos que corresponde a este número de abono - no el id
        DB::table('abonos')->where('abono', '=', $abonado)->update(['ultimo' => 0]);
        $ultimoAbono = DB::table('abonos')->where('abono', '=', $abonado)->orderBy('fechaini', 'desc')->first();
        DB::table('abonos')->where('id', '=', $ultimoAbono->id)->update(['ultimo' => 1]);

        // seleccionamos la última renovación
        $abono = DB::table('abonos')
            ->where('abono', '=', $abonado)
            ->where('ultimo','=',1)
            ->first();
        return $abono ;
    }



    public function generarTicket(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $abono_id = $datos['abono_id'] ? $datos['abono_id'] : 0;
        $cliente_id = $datos['cliente_id'] ? $datos['cliente_id'] : 0;
        $producto_id = $datos['producto_id'] ? $datos['producto_id'] : 0;
        $total = ($datos['total']) ? $datos['total'] : 0;
        $nombreJugador = ($datos['nombre_jugador']) ? $datos['nombre_jugador'] : '';
        $numeroTicket = $this->numeroTicket();

        $datosTicket = [
            'doc'=>$numeroTicket,
            'fecha'=>date("Y-m-d"),
            'cliente_id' => $cliente_id,
            'abono_id' => $abono_id,
            'total' => $total,
            'cobrado' => 0,
            'pendiente'=>$total,
            'estado'=>'CE',
            'notas'=>''
        ];
        // todoevo Leer de configuración id cliente y serie por defecto
        try{
            $registro = new Tickets;
            $registro->saveData($datosTicket);
            $resultado = $registro->id;
        } catch (\Exception $e) {
            $resultado = 0 ;
            $mensajes = $e->getMessage() ;
        }
        // leo el albarán y

        if ($resultado > 0 ) {
            $idTicket = $resultado ;
            // linea que generamos
            $datosLinea = [
                'ticket_id'=>$idTicket,
                'total' => $datos['total'],
                'doc' =>$numeroTicket ,
                'descripcion' => $datos['descripcion'],
                'producto_id' => $producto_id,
                'tarifa_id' => $datos['tarifa_id'],
                'producto' => $datos['producto'],
                'nombre_jugador'=>$datos['nombre_jugador'],
                'cantidad' => 1,
                'precio' => $datos['precio'],
                'es_reserva' => 0,
                'tipo_iva' => 21,
                'notas'=>$datos['notas']
            ];
            $lineaVenta = new LineasVenta();
            $lineaVenta->saveData($datosLinea);
        }

        // todoevo actualizar el abono con el número de ticket y el documento.

        $ticket = Tickets::where('id',$registro->id)->first();
        DB::table('abonos')->where('id', '=', $abono_id)->update(['ticket_id' => $idTicket,'doc'=>$numeroTicket]);

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok', 'datos'=>$ticket);
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }
    public function generarAlbaran(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $abono_id = $datos['abono_id'] ? $datos['abono_id'] : 0;
        $cliente_id = $datos['cliente_id'] ? $datos['cliente_id'] : 0;
        $producto_id = $datos['producto_id'] ? $datos['producto_id'] : 0;
        $total = ($datos['total']) ? $datos['total'] : 0;
        $nombreJugador = ($datos['nombre_jugador']) ? $datos['nombre_jugador'] : '';
        $numeroAlbaran = $this->numeroAlbaran();

        $datosAlbaran = [
            'doc'=>$numeroAlbaran,
            'fecha'=>date("Y-m-d"),
            'cliente_id' => $cliente_id,
            'total' => $total,
            'cobrado' => 0,
            'pendiente'=>$total,
            'notas'=>''
        ];

        // todoevo Leer de configuración id cliente y serie por defecto


        try{
            $registro = new Albaranes;
            $registro->saveData($datosAlbaran);
            $resultado = $registro->id;
        } catch (\Exception $e) {
            $resultado = 0 ;
            $mensajes = $e->getMessage() ;
        }
        // leo el albarán y

        if ($resultado > 0 ) {
            $idAlbaran = $resultado ;
                // linea que generamos
                $datosLinea = [
                    'albaran_id'=>$idAlbaran,
                    'total' => $datos['total'],
                    'doc' =>$numeroAlbaran ,
                    'descripcion' => $datos['descripcion'],
                    'producto_id' => $producto_id,
                    'producto' => $datos['producto'],
                    'cantidad' => 1,
                    'tarifa_id' => $datos['tarifa_id'],
                    'precio' => $datos['precio'],
                    'es_reserva' => 1,
                    'tipo_iva' => 21,
                    'notas'=>$datos['notas'],
                    'nombre_jugador'=>$datos['nombre_jugador'],
                ];
                $lineaVenta = new LineasVenta();
                $lineaVenta->saveData($datosLinea);
        }



        $ticket = Albaranes::where('id',$registro->id)->first();
        DB::table('abonos')->where('id', '=', $abono_id)->update(['albaran_id' => $idAlbaran,'doc'=>$numeroAlbaran]);

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok', 'datos'=>$ticket);
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }

    public function generarAlbaranAgrupado(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $abonos = $datos['abonos'];
        $cliente_id = $datos['cliente_id'] ? $datos['cliente_id'] : 0;
        $idsAbonos = [];
        foreach ($abonos as $abono) {
            $idsAbonos[] = $abono['id'];
        }

        $abonos = Abonos::whereIn('id',$idsAbonos)->get();

        $cliente = DB::table('clientes')->where('id',$cliente_id)->first();

        // Creamos el albarán
        $numeroAlbaran = $this->numeroAlbaran();
        $datosAlbaran = [
            'doc'=>$numeroAlbaran,
            'fecha'=>date("Y-m-d"),
            'cliente_id' => $cliente_id,
            'total' => 0,
            'cobrado' => 0,
            'pendiente'=>0,
            'notas'=>' '
        ];

        try{
            $registro = new Albaranes;
            $registro->saveData($datosAlbaran);
            $resultado = $registro->id;
        } catch (\Exception $e) {
            $resultado = 0 ;
            $mensajes = $e->getMessage() ;
        }
        // leo el albarán y
        $total = 0 ;
        if ($resultado > 0 ) {
            $idAlbaran = $resultado ;

            foreach($abonos as $abono) {
                $datosLinea = [
                    'albaran_id'=>$idAlbaran,
                    'total' => $abono->precio ,
                    'doc' =>$numeroAlbaran ,
                    'descripcion' => $abono->descripcion.' Nº.: '.$abono->abono,
                    'producto_id' => $abono->producto_id,
                    'producto' => $abono->producto,
                    'cantidad' => 1,
                    'tarifa_id' => $abono->tarifa_id,
                    'precio' => $abono->precio,
                    'es_reserva' => 0,
                    'es_abono' => 1,
                    'tipo_iva' => 21,
                    'notas'=>' ',
                    'nombre_jugador'=>$abono->nombre_jugador,
                ];
                $total = $abono->precio + $total ;
                $lineaVenta = new LineasVenta();
                $lineaVenta->saveData($datosLinea);
            }

        }

        $albaran = Albaranes::where('id',$registro->id)->update(['total'=>$total,'pendiente'=>$total]);
        DB::table('abonos')->whereIn('id', $idsAbonos)->update(['albaran_id' => $idAlbaran,'doc'=>$numeroAlbaran]);

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok', 'datos'=>$albaran);
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }


    public function numeroTicket() {
        $ticket_id = 0;
        // todoevo hacerlo multiempresa
        try {
            $record = Contadores::where('doc', 'TICKET')->lockForUpdate()->first();
            $ticket_id = $record->valor;
            $record->valor = $ticket_id + 1 ;
            $record->save();
        } catch (\Exception $e) {
            $ticket_id = 0 ;
        }
        $nuevoReg = 'FS-'.str_pad($ticket_id, 6, "0", STR_PAD_LEFT);
        return $nuevoReg ;
    }

    public function generarSalida(Request $request, Response $response, $args)
    {

        $datos = $request->getParsedBody();

        $id = $datos['id'];
        $abono = DB::table('abonos')->where('id', '=', $id)->first();

        $totalSalidasAbono = DB::table('lineas_venta as l')
            ->leftJoin('tickets as t', 'l.ticket_id', '=', 't.id')
            ->select('l.*', 't.fecha')
            ->where('l.abono_id', '=', $id)
            ->sum('l.cantidad');

        $salidas = !empty($totalSalidasAbono) ? $totalSalidasAbono : 0;
        $abonoActivo = 1;
        if ($salidas >= $abono->gf18 && $abono->anual == 0) {
            $abonoActivo = 0;
        }
        $fechaHoy = date("Y-m-d");
        if ($abono->anual == 1 && ($fechaHoy <= $abono->fechaini || $fechaHoy >= $abono->fechafin )) {
            $abonoActivo = 0;
        }
        DB::table('abonos')->where('id', '=', $abono->id)->update(['activo' => $abonoActivo,'gf18c'=>$salidas]);

        $resultado = 1 ;
        $mensajes = [];

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }
    public function extracto(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $id = $datos['id'];
        $abono = DB::table('abonos')->where('id', '=', $id)->first();
        $salidasAbono = DB::table('lineas_venta as l')
            ->leftJoin('tickets as t', 'l.ticket_id', '=', 't.id')
            ->leftJoin('clientes', 'clientes.id', '=', 't.cliente_id')
            ->select('l.*', 't.fecha','clientes.razon_social')
            ->where('l.abono_id', '=', $id)->get();

        $totalSalidas = 0 ;
        foreach($salidasAbono as $salida) {
            $totalSalidas = $totalSalidas + $salida->cantidad ;
        }
        $resultado = 1 ;
        $mensajes = [];
        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok','salidas'=>$salidasAbono,'abono'=>$abono,'total_salidas'=>$totalSalidas);
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }
        return $response->withJson($respuesta);
    }
    // todoevo pasara a utiles
    public function numeroAlbaran() {
        $nuevoAlbaran = 0;
        try {
            $record = Contadores::where('doc', 'ALBARAN')->lockForUpdate()->first();
            $nuevoAlbaran = $record->valor;
            $record->valor = $nuevoAlbaran + 1 ;
            $record->save();
        } catch (\Exception $e) {
            $nuevoAlbaran = 0 ;
        }
        $nuevoAlbaran = 'ALB-'.str_pad($nuevoAlbaran, 6, "0", STR_PAD_LEFT);
        return $nuevoAlbaran ;
    }

    public function numeroAbono() {
        $nuevoAlbaran = 0;
        try {
            $record = Contadores::where('doc', 'ABONOS')->lockForUpdate()->first();
            $nuevoAlbaran = $record->valor;
            $record->valor = $nuevoAlbaran + 1 ;
            $record->save();
        } catch (\Exception $e) {
            $nuevoAlbaran = 0 ;
        }
        return $nuevoAlbaran ;
    }

}