<?php

namespace App\Controllers;


use Illuminate\Database\Capsule\Manager as DB;
use mPDF;
use AlbaranesCompra ;
use LineasCompra ;
use Contadores ;
use Proveedores ;
use Slim\Views\Twig;
use Slim\Http\Request;
use Slim\Http\Response ;


use \App\BusinessProcess\AlbaranesBusinessProcess as AlbaranesBP ;
use \App\BusinessProcess\ProductosBP as ProductosBP;

class AlbaranesCompraController
{
    // le pasamos lo que necesitamos aunque creo que no es necesario ya que al tener $app tenemos el container
    // esto sería util en caso de que utilizaramos otro contenedor de injection, en ese caso le pasariamos el pimple.

    private $app;
    private $view;

    public function __construct($app, Twig $view)
    {
        // parent::__construct();
        $this->app = $app;
        $this->view = $view;
    }

    public function index()
    {
        echo ':)';
    }

    public function lista(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $cliente_id = ($datos['cliente_id']) ? $datos['cliente_id'] : 0;
        // Pasamos los datos a un array
        // todoevo  esto no es necesario, hay que poner la función como estática.

        $data = AlbaranesCompra::lista($datos['fecha_ini'],$datos['fecha_fin'],'',$cliente_id,false);
        $array = array('resultado' => 'ok','datos' => $data);

        return $response->withJson($array);
    }


    public function grabar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        // Sacamos las lineas del albarán
        $lineas = $datos['lineas'];

        // los cobros y los impuestos ya los hemos generado. Los datos del cliente vienen aparte.

        unset($datos['lineas']);
        unset($datos['impuestos']);
        unset($datos['cobros']);
        unset($datos['totales']);
        //unset($datos['datos_cliente']);

        $registro =  $this->utiles->grabaModelo('Albaranes',$datos);

        $resultado = $registro['id'];
        $mensajes =  $registro['mensajes'];


        // todoevo actualizar albaran al grabar la linea

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }

    public function leer(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $id = ($datos['id']) ? $datos['id'] : 0;
        $albaran = Albaranes::where('id',$id)->first()->toArray();
        $lineasAlbaran = LineasVenta::where('albaran_id',$id)->get()->toArray();
        $cobros = Cobros::where('albaran_id',$id)->get()->toArray();

        $facturaDoc = '' ;

        if ($albaran['factura_id'] > 0) {
            $factura = Facturas::where('id',$albaran['factura_id'])->get();
            $facturaDoc = $factura[0]->doc ;
        }
        $albaran['totales'] = [];
        $albaran['lineas'] = $lineasAlbaran;
        $albaran['cobros'] = $cobros;
        $albaran['factura_doc'] = $facturaDoc ;

        $respuesta = array('resultado' => 'ok', 'datos'=>$albaran);

        return $response->withJson($respuesta);
    }



    public function leerLineasVenta(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();


        $id = ($datos['id']) ? $datos['id'] : 0;
        $lineasAlbaran = LineasVenta::where('albaran_id',$id)->get()->toArray();
        $respuesta = array('resultado' => 'ok', 'datos'=>$lineasAlbaran);

        return $response->withJson($respuesta);
    }

    public function generarAlbaranesReserva(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $reserva_id = ($datos['reserva_id']) ? $datos['reserva_id'] : 0;
        $cliente_id = ($datos['cliente_id']) ? $datos['cliente_id'] : 0;
        $total = ($datos['total']) ? $datos['total'] : 0;
        $nombreReserva = ($datos['nombre_reserva']) ? $datos['nombre_reserva'] : '';
        $referenciaReserva = $datos['referencia_reserva'];
        $datosReserva = ($datos['dreserva']) ? $datos['dreserva'] : array();

        //todoevo comprobar si ya está hecho el albarán.
        // generamos un albaran para cada día.

               foreach($datosReserva as $dreserva) {
                   $numTempAlb = $this->numeroAlbaran();
                   $datosAlb = [
                       'doc'=>$numTempAlb,
                       'fecha'=>$dreserva['fecha'],
                       'cliente_id' => $cliente_id,
                       'reserva_id' => $reserva_id,
                       'referencia_reserva' => $referenciaReserva,
                       'referencia_dreserva' => $dreserva['idreservahora'], // referencia a cada una de las salidas
                       'total' =>  $dreserva['total'],
                       'cobrado' => 0,
                       'pendiente'=> $dreserva['total']
                   ];
                   $registro = new Albaranes;
                   $registro->saveData($datosAlb);
                   $idAlbaran = $registro->id;

                   $conceptoBuggies = '' ;
                   if ($dreserva['buggies'] > 0) {
                       $conceptoBuggies = 'Buggies: '.$dreserva['buggies'];
                   }
                   $conceptoGF = 'Greenfees: ' .$dreserva['jugadores'];
                   // linea que generamos
                   $datosLinea = [
                       'albaran_id'=>$idAlbaran,
                       'fecha_reserva'=>$dreserva['fecha'],
                       'total' => $dreserva['total'],
                       'descripcion' => 'Reserva golf',
                       'producto' => 'RESERVAGOLF',
                       'producto_id' => 0,
                       'cantidad' => 1,
                       'precio' => $dreserva['total'],
                       'concepto_reserva'=>$conceptoGF.' '.$conceptoBuggies,
                       'nombre_jugador'=>$nombreReserva ,
                       'referencia_reserva' =>  $dreserva['idreserva'],
                       'referencia_dreserva' => $dreserva['idreservahora'],
                       'hora_reserva'=>$dreserva['hora'],
                       'recorrido'=> $dreserva['recorrido'],
                       'es_reserva' => 1,
                       'tipo_iva' => 21
                   ];
                   $lineaVenta = new LineasVenta();
                   $lineaVenta->saveData($datosLinea);
               }
        $albaranes = Albaranes::where('reserva_id','=',$reserva_id)->get();
        $respuesta = array('resultado' => 'ok', 'datos'=>$albaranes);

        return $response->withJson($respuesta);
    }


    public function grabarLinea(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $prodBP = new ProductosBP();
        // todoevo actualizar albaran al grabar la linea

        $id = $datos['id'];

        // Cantidad de la linea que tenemos grabada.

        try{
            if ($id > 0) {
                $lineaVenta = LineasVenta::find((int)$id);
                $cantidadAnt = $lineaVenta->cantidad ;
            } else {
                $lineaVenta = new LineasVenta();
                $cantidadAnt = 0 ;
            }
            $resultado = $lineaVenta->saveData($datos);
            $this->actualizaAlbaran($datos['albaran_id']);

            $prodBP->actStock($lineaVenta,$cantidadAnt);


        } catch (\Exception $e) {
            $resultado = 0 ;
            $mensajes = $e->getMessage() ;
        }

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }
    public function borrarLinea(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $prodBP = new ProductosBP();

        $id = $datos['id'];
        try{
            $lineaVenta = LineasVenta::find((int)$id);
            $cantidadAnt = $lineaVenta->cantidad ;
            $resultado = $lineaVenta->delete();
        } catch (\Exception $e) {
            $resultado = 0 ;
            $mensajes = $e->getMessage() ;
        }

        $prodBP->actStock($lineaVenta,$cantidadAnt,'B');

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }

    public function nuevo(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $reserva_id = $datos['reserva_id'] ? $datos['reserva_id'] : 0;
        $cliente_id = $datos['cliente_id'] ? $datos['cliente_id'] : 0;



        $total = ($datos['total']) ? $datos['total'] : 0;
        $nombreReserva = ($datos['nombre_reserva']) ? $datos['nombre_reserva'] : '';
        $referenciaReserva = $datos['referencia_reserva'];
        $datosReserva = ($datos['dreserva']) ? $datos['dreserva'] : array();

        $numTempAlb = $this->numeroAlbaran();

        $datos = [
            'doc'=>$numTempAlb,
            'fecha'=>date("Y-m-d"),
            'cliente_id' => $cliente_id,
            'reserva_id' => $reserva_id,
            'referencia_reserva' => $referenciaReserva,
            'total' => $total,
            'cobrado' => 0,
            'pendiente'=>$total
        ];
        // todoevo Leer de configuración id cliente y serie por defecto

        try{
            $registro = new Albaranes;
            $registro->saveData($datos);
            $resultado = $registro->id;
        } catch (\Exception $e) {
            $resultado = 0 ;
            $mensajes = $e->getMessage() ;
        }
        // leo el albarán y

        if ($resultado > 0 && $datosReserva) {
            // todoevo, traer la referencia de la reserva
            $idAlbaran = $resultado ;
            foreach($datosReserva as $dreserva) {
                $conceptoBuggies = '' ;
                if ($dreserva['buggies'] > 0) {
                    $conceptoBuggies = 'Buggies: '.$dreserva['buggies'];
                }
                $conceptoGF = 'Greenfees: ' .$dreserva['jugadores'];
                // linea que generamos
                $datos = [
                    'albaran_id'=>$idAlbaran,
                    'fecha_reserva'=>$dreserva['fecha'],
                    'total' => $dreserva['total'],
                    'descripcion' => 'Reserva golf',
                    'producto_id' => 0,
                    'producto' => 'RESERVAGOLF',
                    'cantidad' => 1,
                    'precio' => $dreserva['total'],
                    'concepto_reserva'=>$conceptoGF.' '.$conceptoBuggies,
                    'referencia_reserva' =>  $dreserva['idreserva'],
                    'nombre_jugador'=>$nombreReserva ,
                    'hora_reserva'=>$dreserva['hora'],
                    'recorrido'=> $dreserva['recorrido'],
                    'es_reserva' => 1,
                    'tipo_iva' => 21
                ];
                $lineaVenta = new LineasVenta();
                $lineaVenta->saveData($datos);
            }
        }

        $albaran = Albaranes::where('id',$registro->id)->first()->toArray();

        $albaran['totales'] = [];
        $albaran['lineas'] = [];
        $albaran['cobros'] = [];

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok', 'datos'=>$albaran);
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }
    public function nuevoNoReserva(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $reserva_id = 0;
        $cliente_id = 0;
        $total = 0;
        $nombreReserva =  '';
        $referenciaReserva = '';

        $numTempAlb = $this->numeroAlbaran();

        $datos = [
            'doc'=>$numTempAlb,
            'fecha'=>date("Y-m-d"),
            'cliente_id' => $cliente_id,
            'reserva_id' => $reserva_id,
            'referencia_reserva' => $referenciaReserva,
            'total' => $total,
            'cobrado' => 0,
            'pendiente'=>$total
        ];
        // todoevo Leer de configuración id cliente y serie por defecto

        try{
            $registro = new Albaranes;
            $registro->saveData($datos);
            $resultado = $registro->id;
        } catch (\Exception $e) {
            $resultado = 0 ;
            $mensajes = $e->getMessage() ;
        }
        // leo el albarán y
        $albaran = $registro ;
        $albaran['totales'] = [];
        $albaran['lineas'] = [];
        $albaran['cobros'] = [];

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok', 'datos'=>$albaran);
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }
    public function cobros(Request $request, Response $response, $args)
    {
        $parametros = $request->getParsedBody();
        $idAlbaran = $parametros['id'];
        $data = DB::table('cobros')
            ->leftJoin('formaspago', 'formaspago.id', '=', 'cobros.formapago_id')
            ->select('cobros.*','formaspago.descripcion','formaspago.tipo')
            ->where('cobros.albaran_id','=',$idAlbaran) ;
        $data->orderBy('cobros.fecha');
        $datos = $data->get();
        $respuesta = array('datos'=>$datos);
        return $response->withJson($respuesta);
    }

    /*
     * En borrar solamente podremos hacerlo si el albarán no está cobrado o facturado.
     */
    public function borrar(Request $request, Response $response, $args)
    {
        $parametros = $request->getParsedBody();
        $id = $parametros['id'];
        $albaran = Albaranes::where('id',$id)->get();
        $respuesta = array('resultado' => 'ok','albaran'=>$albaran);
        echo json_encode($respuesta,JSON_NUMERIC_CHECK);
    }
    public function numeroAlbaran() {
        $nuevoAlbaran = 0;
        // todoevo hacerlo multiempresa
        try {
                $record = Contadores::where('doc', 'ALBARAN')->lockForUpdate()->first();
                $nuevoAlbaran = $record->valor;
                $record->valor = $nuevoAlbaran + 1 ;
                $record->save();
        } catch (\Exception $e) {
            $nuevoAlbaran = 0 ;
        }
        $nuevoAlbaran = 'ALB-'.str_pad($nuevoAlbaran, 6, "0", STR_PAD_LEFT);
        return $nuevoAlbaran ;
    }


    public function imprimir(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $id = ($datos['id']) ? $datos['id'] : 0;
        $albaran = Albaranes::where('id',$id)->first()->toArray();
        $lineasAlbaran = LineasVenta::where('albaran_id',$id)->get()->toArray();
        $cliente = Clientes::where('id',$albaran['cliente_id'])->get()->toArray();
        $albaran['totales'] = [];
        $albaran['lineas'] = $lineasAlbaran;
        $albaran['cliente'] = $cliente[0];


        $totalFactura = 0;
        $impuestos = array();
        $datosImpuesto = array();


        // Procedimiento para calcular las bases del iva incluido en los albaranes.
        if ($lineasAlbaran) {
            foreach ($lineasAlbaran as $linea) {
                $porcentage = $linea['tipo_iva'];
                if (!isset($impuestos[$porcentage])) {
                   $impuestos[$porcentage] = ['total_iva' => $porcentage, 'base'=> 0,'total'=>0,'tipo_iva'=>$porcentage];
                }
                $impuestos[$porcentage]['total'] = $impuestos[$porcentage]['total'] + $linea['total'];
            };
        }
        foreach ($impuestos as $impuesto) {
            $base = $impuesto['total'] / ( 1 + ($impuesto['tipo_iva'] / 100) ) ;
            $base = round($base,2);
            $impuestos[$porcentage]['total_iva'] = $impuesto['total'] - $base ;
            $impuestos[$porcentage]['base'] = $base;
        };
        $albaran['impuestos'] = $impuestos;


        // renderizamos el html

        $html = $this->app->view->fetch('albaran.twig', ['doc' => $albaran]);
        $mpdf = new mPDF('c', 'A4', 0, 'Arial', 5, 5, 5, 5, 5, 5);

        $mpdf->WriteHTML($html);
        $nombreFichero = 'ficherostemp3duadjfa/'.$albaran['doc'].'-'.$albaran['referencia_reserva'].'.pdf';
        $mpdf->Output($nombreFichero, 'F');
        $respuesta = array('link' => $nombreFichero);
        return $response->withJson($respuesta);
    }


    public function imprimirListado(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $cliente_id = ($datos['cliente_id']) ? $datos['cliente_id'] : 0;

        $albaranes = new Albaranes();
        $data = $albaranes->lista($datos['fecha_ini'],$datos['fecha_fin'],'',$cliente_id,false);

        $html = $this->app->view->fetch('listado-albaranes-por-fecha.twig', ['albs' => $data]);

        $mpdf = new mPDF('c', 'A4', 0, 'Arial', 5, 5, 5, 5, 5, 5);

        $mpdf->WriteHTML($html);
        $nombreFichero = 'ficherostemp3duadjfa/'.'listado-albaranes-x-fecha.pdf';
        $mpdf->Output($nombreFichero, 'F');
        $respuesta = array('link' => $nombreFichero,'datos'=>$data);
        return $response->withJson($respuesta);
    }

    public function facturar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $albaranes  = $datos['albaranes'];
        $fechaFac   = $datos['fecha'];
        $cliente_id = $datos['cliente_id'];
        $albaranesFac = [];
        $factura = [];


        $ano = ($datos['ano']) ? $datos['ano'] : 0;
        $serie = ($datos['serie']) ? $datos['serie'] : '01';

        // comprobamos la fecha de la facturación.

        $data = DB::table('facturas')
            ->leftJoin('series', 'series.id', '=', 'facturas.serie_id')
            ->select('facturas.fecha','series.codigo as serie_factura')
            ->whereYear('facturas.fecha', '=', $ano)
            ->where('series.codigo','=',$serie)
            ->orderBy('facturas.fecha','desc');
        $ultimaFactura = $data->first(); // devuelve un objeto



        if (isset($ultimaFactura) && $ultimaFactura->fecha > $fechaFac) {
            $yyyy = substr($ultimaFactura->fecha ,0,4);
            $mm = substr($ultimaFactura->fecha ,5,2);
            $dd = substr($ultimaFactura->fecha ,8,2);
            $fecha = $dd.'-'.$mm.'-'.$yyyy;
            $respuesta = ['estado'=>'ko',
                'mensaje'=>'La fecha no puede ser inferior a la de la última factura',
                'fecha'=>$fecha];
            return $response->withJson($respuesta);
            return ;
        }

        /*
         * La serie tiene que venir de la consultay debemos de seleccionar los albaranes por serie.
         * ojo con el tema de la tour operación. Poner un checkout en los albaranes para que no se puedan facturar.
         */
        // todoevo Control de errores
        if (count($albaranes) > 0) {
            // creo factura con el cliente y la fecha qu eme mandan
            $factura = new Facturas();
            $datosFac = ['cliente_id'=>$cliente_id,'fecha'=>$fechaFac,'serie_id'=>1] ;
            $factura->saveData($datosFac);
            $factura_id = $factura->id ;

            // actualizamos los albaranes y las lineas de venta.

            if ($factura_id > 0) {
                $anoFac = date('Y', strtotime($fechaFac));
                $docFac = $this->numeroFactura($anoFac,'01');
                // actualizo la factura

                Albaranes::whereIn('id', $albaranes)->update(['factura_id' => $factura_id]);
                LineasVenta::whereIn('albaran_id', $albaranes)->update(['factura_id' => $factura_id]);
                Cobros::whereIn('albaran_id', $albaranes)->update(['factura_id' => $factura_id]);

                $totales = Facturas::recalcular($factura_id);
                Facturas::where('id', $factura_id)->update(['doc' => $docFac,'total'=>$totales['total']]);
                $factura = Facturas::where('id', $factura_id)->get();
            }
        }

        // recalculamos la factura


        // leemos la factura.


        // $albaranesFac = Albaranes::whereIn('id', $albaranes)->get();
        $respuesta = ['estado'=>'ok','datos'=>$albaranesFac,'factura'=>$factura,'id'=>$factura_id];
        return $response->withJson($respuesta);
    }





    public function numeroFactura($ano,$serie) {
        $nuevoAlbaran = 0;

        // comprobamos que existe el contador
        $record = Contadores::where('doc', 'FAC-'.$ano.'-'.$serie)->first();

        // si no existe debemos de crearlo.
        if (!$record) {
            $contador = new Contadores();
            $datos = ['doc'=>'FAC-'.$ano.'-'.$serie, 'valor'=>1];
            $contador->saveData($datos);
        }
        // todoevo hacerlo multiempresa
        try {
            $record = Contadores::where('doc', 'FAC-'.$ano.'-'.$serie)->lockForUpdate()->first();
            $nuevoDoc = $record->valor;
            $record->valor = $nuevoDoc + 1 ;
            $record->save();
            $nuevaFac = 'FAC-'.$ano.'-'.$serie.'-'.str_pad($nuevoDoc, 6, "0", STR_PAD_LEFT);
        } catch (\Exception $e) {
            $nuevaFac = '' ;
        }
        return $nuevaFac ;
    }

    public function actualizaAlbaran($id) {
        $lineasAlbaran = LineasVenta::where('albaran_id',$id)->get()->toArray();
        $total = 0;
        // todoevo recalcular los cobros. Esto tambien nos vale para los registros de iva en las facturas
        // y el saldo de los clientes

        if ($lineasAlbaran) {
            foreach ($lineasAlbaran as $linea) {
                $total = $total + $linea['total'];
            };
        }
        try{
            $albaran = Albaranes::find((int)$id);
            $datos = [
              'total' => $total,
              'pendiente' => $total - $albaran->cobrado
            ];
            $resultado = $albaran->saveData($datos);
        } catch (\Exception $e) {
            $resultado = 0 ;
            $mensajes = $e->getMessage() ;
        }
    }


}