<?php

namespace App\Controllers;

use Illuminate\Database\Capsule\Manager as DB;
use Slim\Http\Request;
use Slim\Http\Response;
use Slim\Views\Twig;

class ClientesController
{

    private $app;
    private $view;
    private $utiles;
    private $pdo;

    public function __construct($app, Twig $view)
    {
        $this->app = $app;
        $this->view = $view;
        $container = $this->app->getContainer();
        $this->utiles = $container->get('Utiles');
        $this->pdo = $container->get('Pdo');
    }

    public function index()
    {
        echo ':)';
    }

    public function lista(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $razonsocial = $datos['razonsocial'];
        $id = $datos['id'];

        $data = DB::table('clientes')->orderBy('clientes.razon_social');
        // si buscamos por razón social
        if ($razonsocial) {
            $data->where('clientes.razon_social', 'like', "" . strtoupper($razonsocial) . "%");
        }
        if ($id) {
            $data->where('clientes.id', '=', $id);
        }
        $datosClientes = $data->get();
        $array = array('resultado' => 'ok', 'datos' => $datosClientes);


        return $response->withJson($array);
    }

    public function ver(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $id = $datos['id'];
        $data = DB::table('clientes')->where('clientes.id', '=', $id)->first();
        $array = array('resultado' => 'ok', 'datos' => $data);

        return $response->withJson($array);
    }

    public function grabar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $registro = $this->utiles->grabaModelo('Clientes', $datos);

        $resultado = $registro['resultado'];
        $mensajes = $registro['mensajes'];

        if ($resultado == 'ok') {
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko', 'mensajes' => $mensajes);
        }

        return $response->withJson($respuesta);

    }

    public function extracto(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $cliente_id = $datos['cliente_id'];

        $cliente = DB::table('clientes')
            ->where('clientes.id', '=', $cliente_id)
            ->get();

        // todoevo añadir cobros y facturas

        // albarnes, solamente los facturados.
        $ventasAlbaranes = DB::table('albaranes as a')
            ->leftJoin('clientes as c', 'c.id', '=', 'a.cliente_id')
            ->select('a.*', 'c.razon_social')
            ->where('a.cliente_id', '=', $cliente_id)
            ->where('a.id', '>', 0)
            ->get();

        $ventasTickets = DB::table('tickets as t')
            ->leftJoin('clientes as c', 'c.id', '=', 't.cliente_id')
            ->select('t.*', 'c.razon_social')
            ->where('t.cliente_id', '=', $cliente_id)
            ->where('t.id', '>', 0)
            ->get();

        $cobros = DB::table('cobros as t')
            ->leftJoin('clientes as c', 'c.id', '=', 't.cliente_id')
            ->select('t.*', 'c.razon_social')
            ->where('t.cliente_id', '=', $cliente_id)
            ->where('t.id', '>', 0)
            ->get();

        $depositos = DB::table('depositos as t')
            ->leftJoin('clientes as c', 'c.id', '=', 't.cliente_id')
            ->select('t.*', 'c.razon_social')
            ->where('t.cliente_id', '=', $cliente_id)
            ->where('t.id', '>', 0)
            ->get();
        // todoevo copiar de laurogolf
        $movimientos = [];
        $totalVentas = 0;
        $totalCobros = 0;


        foreach ($ventasTickets as $v) {
            $totalVentas = $totalVentas + $v->total;
            $movimientos[] = array('tipo' => 'VT',
                'fecha' => $v->fecha,
                'total' => $v->total,
                'saldo' => 0,
                'documento' => $v->doc,
                'razon_social' => $v->razon_social);
        }


        foreach ($ventasAlbaranes as $v) {
            $totalVentas = $totalVentas + $v->total;
            $movimientos[] = array('tipo' => 'VA',
                'fecha' => $v->fecha,
                'total' => $v->total,
                'saldo' => 0,
                'documento' => $v->doc,
                'razon_social' => $v->razon_social);
        }

        foreach ($cobros as $v) {
            $totalCobros = $totalCobros + $v->total;
            $movimientos[] = array('tipo' => 'CO',
                'fecha' => $v->fecha,
                'total' => $v->total,
                'saldo' => 0,
                'documento' => $v->doc,
                'razon_social' => $v->razon_social);
        }


        if (!empty($movimientos)) {
            $sort = array();
            foreach ($movimientos as $k => $v) {
                $sort['fecha'][$k] = $v['fecha'];
                $sort['tipo'][$k] = $v['tipo'];
            }
            array_multisort($sort['fecha'], SORT_ASC, $sort['tipo'], SORT_ASC, $movimientos);
            $saldo = 0;
            foreach ($movimientos as $k => $v) {

                if ($v['tipo'] == 'CO') {
                    $saldo = $saldo + $v['total'];
                }
                if ($v['tipo'] == 'VT' || $v['tipo'] == 'VA') {
                    $saldo = $saldo - $v['total'];
                }
                $movimientos[$k]['saldo'] = $saldo;
            }
        }


        $datos = [
            'movimientos' => $movimientos,
            'total_ventas' => $totalVentas,
            'total_cobros' => $totalCobros,
            'saldo' => $saldo,
            'cliente' => $cliente
        ];
        $resultado = array('resultado' => 'ok', 'datos' => $datos);
        return $response->withJson($resultado);
    }


    public function borrar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $id = $datos['id'];
        $tieneMovimientos = 1;

        $albaranes = DB::table('albaranes as d')->where('d.cliente_id', '=', $id)->count();
        $tickets = DB::table('tickets as d')->where('d.cliente_id', '=', $id)->count();
        $facturas = DB::table('tickets as d')->where('d.cliente_id', '=', $id)->count();

        if ($albaranes > 0 || $tickets > 0 || $facturas > 0) {
            $tieneMovimientos = 1;
        } else {
            $tieneMovimientos = 0;
        }
        if ($tieneMovimientos == 0) {
            $data = DB::table('clientes')->where('id', '=', $id)->delete();
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko', 'tienemovimientos' => $tieneMovimientos);
        }

        return $response->withJson($respuesta);

    }


    // Código para las cuotas de los socios.

    public function listaCuotas(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $id = $datos['id'];
        $data = DB::table('cuotas_socios')->where('cliente_id', '=', $id);
        $datos = $data->get();
        $resultado = array('resultado' => 'ok', 'datos' => $datos);
        return $response->withJson($resultado);
    }

    public function grabaCuota(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $registro = $this->utiles->grabaModelo('CuotasSocios', $datos);
        $resultado = $registro['resultado'];
        $mensajes = $registro['mensajes'];
        if ($resultado == 'ok') {
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko', 'mensajes' => $mensajes);
        }
        return $response->withJson($respuesta);

    }
}