<?php

namespace App\Controllers;

use Illuminate\Database\Capsule\Manager as DB;
use Cobros ;
use LineasVenta ;
use Contadores ;
use Slim\Views\Twig;
use Slim\Http\Request;
use Slim\Http\Response ;


class CobrosController
{

    private $app;
    private $view;
    private $utiles ;
    private $pdo ;

    public function __construct($app, Twig $view)
    {
        $this->app = $app;
        $this->view = $view;
        $container = $this->app->getContainer();
        $this->utiles = $container->get('Utiles') ;
        $this->pdo = $container->get('Pdo') ;
    }
    public function index()
    {
        echo ':)';
    }

    public function lista(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $cliente_id = (isset($datos['cliente_id'])) ? $datos['cliente_id'] : '' ;
        $fechaIni =  $datos['fecha_ini'];
        $fechaFin =  $datos['fecha_fin'];

        $data = DB::table('cobros as c')
            ->leftJoin('clientes', 'clientes.id', '=', 'c.cliente_id')
            ->leftJoin('formaspago as fp', 'fp.id', '=', 'c.formapago_id')
            ->select('c.*','clientes.razon_social','fp.descripcion','fp.tipo')
            ->whereBetween('c.fecha', array($fechaIni,$fechaFin)) ;
        if($cliente_id > 0) {
            $data->where('c.cliente_id','=',$cliente_id);
        }
        $data->orderBy('c.fecha');
        $datos = $data->get();
        $respuesta = array('resultado' => 'ok','datos'=>$datos);


        return $response->withJson($respuesta);
    }

    public static function listaFacturados($factura_id) {
        $data = DB::table('albaranes')
            ->leftJoin('clientes', 'clientes.id', '=', 'albaranes.cliente_id')
            ->leftJoin('facturas', 'facturas.id', '=', 'albaranes.factura_id')
            ->leftJoin('reservas', 'reservas.id', '=', 'albaranes.reserva_id')
            ->select('albaranes.*','clientes.razon_social','reservas.idreserva','reservas.nombre','facturas.doc as factura_doc');
        $data->where('albaranes.factura_id','=',$factura_id);
        $data->orderBy('albaranes.fecha');
        return $data->get();
    }


    public function grabar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        // Sacamos las lineas del albarán

        $registro = $this->utiles->grabaModelo('Cobros',$datos);

        $resultado = $registro['id'];
        $mensajes =  $registro['mensajes'];
        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }

    public function borrar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        // Sacamos las lineas del albarán
        Cobros::where('id',$datos['id'])->delete(); // todoevo control deerrores
        $respuesta = array('resultado' => 'ok');
        return $response->withJson($respuesta);
    }

}