<?php

namespace App\Controllers;

use Illuminate\Database\Capsule\Manager as DB;
use Depositos ;
use Slim\Views\Twig;
use Slim\Http\Request;
use Slim\Http\Response ;


class DepositosController
{
    private $app;
    private $view;

    public function __construct($app, Twig $view)
    {
        // parent::__construct();
        $this->app = $app;
        $this->view = $view;
    }

    public function index()
    {
        echo ':)';
    }

    public function lista(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $cliente_id = (isset($datos['cliente_id'])) ? $datos['cliente_id'] : '' ;
        $fechaIni =  $datos['fecha_ini'];
        $fechaFin =  $datos['fecha_fin'];

        $data = DB::table('depositos as d')
            ->leftJoin('clientes', 'clientes.id', '=', 'd.cliente_id')
            ->leftJoin('formaspago as fp', 'fp.id', '=', 'd.formapago_id')
            ->select('d.*','clientes.razon_social','fp.descripcion','fp.tipo')
            ->whereBetween('d.fecha', array($fechaIni,$fechaFin)) ;
        if($cliente_id > 0) {
            $data->where('d.cliente_id','=',$cliente_id);
        }
        $data->orderBy('d.fecha');
        $datos = $data->get();
        $respuesta = array('resultado' => 'ok','datos'=>$datos);


        return $response->withJson($respuesta);
    }



    public function listaPendientes(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $cliente_id = (isset($datos['cliente_id'])) ? $datos['cliente_id'] : '' ;
        $data = DB::table('depositos as d')
            ->leftJoin('formaspago as fp', 'fp.id', '=', 'd.formapago_id')
            ->select('d.*','fp.descripcion','fp.tipo')
            ->where('d.cliente_id','=',$cliente_id)
            ->where('d.saldo',">",0)
            ->orderBy('d.fecha');
        $datos = $data->get();
        $respuesta = array('resultado' => 'ok','datos'=>$datos);


        return $response->withJson($respuesta);
    }


    public function grabar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $registro =  $this->utiles->grabaModelo('Depositos',$datos);
        $resultado = $registro['id'];
        $mensajes =  $registro['mensajes'];



        if ($resultado > 0) {
            $this->recalcularDeposito($resultado);
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }
        return $response->withJson($respuesta);
    }


    public function asignarDepositoFactura(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $resultado = 0 ;

        $deposito_id    = $datos['deposito_id'];
        $factura_id     = $datos['factura_id'];
        $cliente_id     = $datos['cliente_id'];
        $total          = $datos['total'];
        $fecha          = date("Y-m-d") ; // fecha de hoy

        $this->recalcularDeposito($deposito_id);
        // todoevo comprobamos que el saldo es correcto
        $deposito = DB::table('depositos')->where('id', '=', $deposito_id)->first();

        $mensajes = [];
        $error = 0 ;

        if  ($total > $deposito->saldo) {
            $mensajes[] = 'El total es mayor que el saldo pendiente de asignar de este depósito' ;
            $error = 1 ;
        }
        if ($error == 0 ) {
            $resultado = DB::table('depositos_asignaciones')->insert(
                [ 'deposito_id' => $deposito_id,
                    'factura_id' => $factura_id,
                    'total' => $total,
                    'fecha' => $fecha
                ]
            );
            $this->recalcularDeposito($deposito_id);
        }

        if ($error == 0) {
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }
        return $response->withJson($respuesta);
    }

    public function ver(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $id = $datos['id'];
        $data = DB::table('depositos')->where('id', '=', $id)->first();
        $respuesta = array('resultado' => 'ok','datos' => $data);
        return $response->withJson($respuesta);
    }


    public function verAsignaciones(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $id = $datos['id'];
        $data = DB::table('depositos_asignaciones as d')
            ->leftJoin('facturas as f', 'f.id', '=', 'd.factura_id')
            ->select('d.*','f.doc')
            ->where('d.deposito_id', '=', $id)->get();
        $respuesta = array('resultado' => 'ok','datos' => $data);
        return $response->withJson($respuesta);
    }

    public function borrar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        // Sacamos las lineas del albarán
        // todoevo comprobar si tiene el saldo = total. Es decir, no tiene asignado el dinero.
        Depositos::where('id',$datos['id'])->delete(); // todoevo control deerrores

        $respuesta = array('resultado' => 'ok');


        return $response->withJson($respuesta);
    }

    public function borrarAsignacion(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $id     = $datos['id'];
        $deposito_id  = $datos['deposito_id'];
        $data = DB::table('depositos_asignaciones')->where('id', '=', $id)->delete();

        $this->recalcularDeposito($deposito_id);

        $respuesta = array('resultado' => 'ok');
        return $response->withJson($respuesta);
    }

    public function recalcularDeposito($id) {
        $deposito = DB::table('depositos')
            ->where('id',$id)
            ->first();

        $total = $deposito->total ;

        $totalAsignado = DB::table('depositos_asignaciones as d')
            ->where('d.deposito_id','=',$id)
            ->sum('d.total') ;

        $deposito = DB::table('depositos')
            ->where('id',$id)
            ->update([  'saldo' => $total - $totalAsignado,
                        'asignado' => $totalAsignado ]);
    }

}