<?php

namespace App\Controllers;

use Slim\Views\Twig;
use Slim\Http\Request;
use Slim\Http\Response ;

use App\BusinessProcess\ProductosBP as ProductosBP;
use Contadores;
use FacturasCompra;
use Illuminate\Database\Capsule\Manager as DB;
use LineasCompra;
use mPDF;

class FacturasCompraController
{

    private $app;
    private $view;
    private $utiles;
    private $pdo;

    public function __construct($app, Twig $view)
    {
        $this->app = $app;
        $this->view = $view;
        $container = $this->app->getContainer();
        $this->utiles = $container->get('Utiles');
        $this->pdo = $container->get('Pdo');
    }


    public function lista(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $proveedor_id = (isset($datos['proveedor_id'])) ? $datos['proveedor_id'] : 0;
        // Pasamos los datos a un array
        $data = FacturasCompra::lista($datos['fecha_ini'], $datos['fecha_fin'], '', $proveedor_id);

        $respuesta = array('resultado' => 'ok', 'datos' => $data);
        return $response->withJson($respuesta);
    }

    public function grabar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        // los cobros y los impuestos ya los hemos generado. Los datos del cliente vienen aparte.
        unset($datos['lineas']);
        unset($datos['impuestos']);
        unset($datos['totales']);

        $registro = $this->utiles->grabaModelo('FacturasCompra', $datos);

        $resultado = $registro['id'];
        $mensajes = $registro['mensajes'];

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko', 'mensajes' => $mensajes);
        }

        return $response->withJson($respuesta);
    }

    public function leer(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $id = (isset($datos['id'])) ? $datos['id'] : 0;
        $factura = FacturasCompra::where('id', $id)->first()->toArray();


        $data = DB::table('lineas_compra as l')
            ->leftJoin('facturas_compra as f', 'f.id', '=', 'l.factura_id')
            ->select('l.*', 'f.fecha')
            ->where('l.factura_id', '=', $id);
        $lineas = $data->get();

        $factura['totales'] = [];
        $factura['lineas'] = $lineas;

        $respuesta = array('resultado' => 'ok', 'datos' => $factura);

        return $response->withJson($respuesta);
    }

    public function leerLineasCompra(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $id = (isset($datos['id'])) ? $datos['id'] : 0;
        // $lineasAlbaran = LineasVenta::where('factura_id',$id)->get()->toArray();

        $data = DB::table('lineas_compra as l')
            ->select('l.*')
            ->where('l.factura_id', '=', $id);
        // $data->orderBy('l.fecha');
        $lineasAlbaran = $data->get();

        $respuesta = array('resultado' => 'ok', 'datos' => $lineasAlbaran);
        return $response->withJson($respuesta);
    }


    /*
     * Procedimiento para meter facturas directamente sin pasar por el mantenimiento de albaranes.
     * O la hacemos directa y o hacemos una temporal y luego asignamos el número definitivo pero
     * tenemos que comprobar la fecha para que cumpla con la legalidad.
     */
    public function nuevo(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $proveedor_id = (isset($datos['proveedor_id'])) ? $datos['proveedor_id'] : 0;
        $total = (isset($datos['total'])) ? $datos['total'] : 0;

        // numero temporal de factura.
        $numTemp = 'FAP-' . $this->utiles->generaIdReserva();

        $datos = [
            'doc' => $numTemp,
            'fecha' => date("Y-m-d"),
            'proveedor_id' => $proveedor_id,
            'total' => $total
        ];

        try {
            $registro = new FacturasCompra;
            $registro->saveData($datos);
            $resultado = $registro->id;
        } catch (\Exception $e) {
            $resultado = 0;
            $mensajes = $e->getMessage();
        }
        // leo el albarán y

        $documento = FacturasCompra::where('id', $registro->id)->first()->toArray();

        $documento['totales'] = [];
        $documento['lineas'] = [];

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok', 'datos' => $documento);
        } else {
            $respuesta = array('resultado' => 'ko', 'mensajes' => $mensajes);
        }
        return $response->withJson($respuesta);
    }


    public function numeroFactura($ano, $serie)
    {
        // comprobamos que existe el contador
        $record = Contadores::where('doc', 'FAP-' . $ano . '-' . $serie)->first();
        // si no existe debemos de crearlo.
        if (!$record) {
            $contador = new Contadores();
            $datos = ['doc' => 'FAP-' . $ano . '-' . $serie, 'valor' => 1];
            $contador->saveData($datos);
        }
        // todoevo hacerlo multiempresa
        try {
            $record = Contadores::where('doc', 'FAP-' . $ano . '-' . $serie)->lockForUpdate()->first();
            $nuevoDoc = $record->valor;
            $record->valor = $nuevoDoc + 1;
            $record->save();
            $nuevaFac = 'FAP-' . $ano . '-' . $serie . '-' . str_pad($nuevoDoc, 6, "0", STR_PAD_LEFT);
        } catch (\Exception $e) {
            $nuevaFac = '';
        }
        return $nuevaFac;
    }


    public function verUltima(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $ano = ($datos['ano']) ? $datos['ano'] : 0;
        $serie = ($datos['serie']) ? $datos['serie'] : '01';
        $data = DB::table('facturas_compra as f')
            ->leftJoin('series', 'series.id', '=', 'f.serie_id')
            ->select('f.fecha', 'series.codigo as serie_factura')
            ->whereYear('f.fecha', '=', $ano)
            ->where('series.codigo', '=', $serie)
            ->orderBy('f.fecha', 'desc');
        $ultimaFactura = $data->first();
        $respuesta = array('datos-ultima-factura' => $ultimaFactura);
        return $response->withJson($respuesta);
    }

    public function grabarLinea(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $prodBP = new ProductosBP();

        $id = $datos['id'];

        // Cantidad de la linea que tenemos grabada.

        try {
            if ($id > 0) {
                $lineaCompra = LineasCompra::find((int)$id);
                $cantidadAnt = $lineaCompra->cantidad;
            } else {
                $lineaCompra = new LineasCompra();
                $cantidadAnt = 0;
            }
            $resultado = $lineaCompra->saveData($datos);


        } catch (\Exception $e) {
            $resultado = 0;
            $mensajes = $e->getMessage();
        }

        if ($resultado > 0) {
            $prodBP->actStockProducto($datos['producto_id']);
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko', 'mensajes' => $mensajes);
        }

        return $response->withJson($respuesta);
    }

    public function borrarLinea(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $prodBP = new ProductosBP();
        $id = $datos['id'];
        try {
            $lineaCompra = LineasCompra::find((int)$id);
            $cantidadAnt = $lineaCompra->cantidad;
            $resultado = $lineaCompra->delete();
        } catch (\Exception $e) {
            $resultado = 0;
            $mensajes = $e->getMessage();
        }

        // $prodBP->actStockCompras($lineaCompra,$cantidadAnt,'B');

        if ($resultado > 0) {
            $prodBP->actStockProducto($lineaCompra->producto_id);
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko', 'mensajes' => $mensajes);
        }

        return $response->withJson($respuesta);
    }


    /* public function anaAlbaranesFactura() {
         $datos = toArray(json_decode($this->app->request()->getBody()));
         $albaranes  = $datos['albaranes'];
         $factura_id = $datos['factura_id'];

         $albaranesFac = [];
         $factura = [];


         if (count($albaranes) > 0) {
             // creo factura con el cliente y la fecha qu eme mandan

             $factura = Facturas::where('id',$factura_id)->first();
             // actualizamos los albaranes y las lineas de venta.

             if ($factura_id > 0) {
                 // actualizo la factura
                 Albaranes::whereIn('id', $albaranes)->update(['factura_id' => $factura_id]);
                 LineasVenta::whereIn('albaran_id', $albaranes)->update(['factura_id' => $factura_id]);
                 Cobros::whereIn('albaran_id', $albaranes)->update(['factura_id' => $factura_id]);

                 // recalculamos el total de la factura
                 $totales = Facturas::recalcular($factura_id);
                 Facturas::where('id', $factura_id)->update(['total'=>$totales['total']]);
                 $factura = Facturas::where('id', $factura_id)->get();
             }
         }

         // recalculamos la factura
         // leemos la factura.
         // $albaranesFac = Albaranes::whereIn('id', $albaranes)->get();
         $respuesta = ['estado'=>'ok','datos'=>$albaranesFac,'factura'=>$factura,'id'=>$factura_id];
         echo json_encode($respuesta,JSON_NUMERIC_CHECK);
     }

     public function quitarAlbaranesFactura() {
         $datos = toArray(json_decode($this->app->request()->getBody()));
         $albaranes  = $datos['albaranes'];
         $factura_id = $datos['factura_id'];

         $albaranesFac = [];
         $factura = [];


         if (count($albaranes) > 0) {
             // creo factura con el cliente y la fecha qu eme mandan

             $factura = Facturas::where('id',$factura_id)->first();
             // actualizamos los albaranes y las lineas de venta.

             if ($factura_id > 0) {
                 // actualizo la factura
                 Albaranes::whereIn('id', $albaranes)->update(['factura_id' => 0]);
                 LineasVenta::whereIn('albaran_id', $albaranes)->update(['factura_id' =>0]);
                 Cobros::whereIn('albaran_id', $albaranes)->update(['factura_id' => 0]);

                 // recalculamos el total de la factura
                 $totales = Facturas::recalcular($factura_id);
                 Facturas::where('id', $factura_id)->update(['total'=>$totales['total']]);
                 $factura = Facturas::where('id', $factura_id)->get();
             }
         }

         // recalculamos la factura
         // leemos la factura.
         // $albaranesFac = Albaranes::whereIn('id', $albaranes)->get();
         $respuesta = ['estado'=>'ok','datos'=>$albaranesFac,'factura'=>$factura,'id'=>$factura_id];
         echo json_encode($respuesta,JSON_NUMERIC_CHECK);
     }*/
    public function borrar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $id = $datos['id'];

        $data = DB::table('lineas_compra as l')->select('l.*')->where('l.factura_id', '=', $id);
        $lineasFactura = $data->get();

        $data = DB::table('lineas_compra')->where('factura_id', '=', $id)->delete();
        $data = DB::table('facturas_compra')->where('id', '=', $id)->delete();

        $prodBP = new ProductosBP();
        foreach ($lineasFactura as $linea) {
            $prodBP->actStockProducto($linea->producto_id);
        }

        $mensajes = [];

        $resultado = 1;
        if ($resultado > 0) {

            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko', 'mensajes' => $mensajes);
        }

        return $response->withJson($respuesta);
    }

}