<?php

namespace App\Controllers;

use Illuminate\Database\Capsule\Manager as DB;
use Albaranes ;
use Facturas ;
use LineasVenta ;
use Contadores ;
use Clientes ;
use mPDF ;


use Slim\Views\Twig;
use Slim\Http\Request;
use Slim\Http\Response ;


class FacturasController
{

    private $app;
    private $view;
    private $utiles;
    private $pdo;

    public function __construct($app, Twig $view)
    {
        $this->app = $app;
        $this->view = $view;
        $container = $this->app->getContainer();
        $this->utiles = $container->get('Utiles');
        $this->pdo = $container->get('Pdo');
    }


    public function index()
    {
        echo ':)';
    }

    public function lista(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $cliente_id = ($datos['cliente_id']) ? $datos['cliente_id'] : 0;
        // Pasamos los datos a un array
        $data = Facturas::lista($datos['fecha_ini'],$datos['fecha_fin'],'',$cliente_id);
        $array = array('resultado' => 'ok','datos' => $data);

        return $response->withJson($array);
    }

    public function listaPowerBi(Request $request, Response $response, $args)
    {
        $data = DB::table('lineas_venta')
            ->leftJoin('albaranes', 'albaranes.id', '=', 'lineas_venta.albaran_id')
            ->select('lineas_venta.*','albaranes.doc as albaran_doc','albaranes.fecha as albaran_fecha');
        $data->orderBy('albaran_doc');
        $datos = $data->get();


        return $response->withJson($datos);

    }

    public function grabar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        // Sacamos las lineas del albarán

        // los cobros y los impuestos ya los hemos generado. Los datos del cliente vienen aparte.
        unset($datos['lineas']);
        unset($datos['impuestos']);
        unset($datos['cobros']);
        unset($datos['depositos']);
        unset($datos['totales']);

        $registro =  $this->utiles->grabaModelo('Facturas',$datos);

        $resultado = $registro['id'];
        $mensajes =  $registro['mensajes'];

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }

    public function imprimirListado(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $cliente_id = ($datos['cliente_id']) ? $datos['cliente_id'] : 0;
        $data = Facturas::lista($datos['fecha_ini'],$datos['fecha_fin'],'',$cliente_id);

        $facturas =[];

        foreach ($data as $factura) {
            $impuestos = Facturas::recalcular($factura->id);
            $facturas[] = array('fac' => $factura, 'impuestos' => $impuestos['impuestos']);
        }

        $html = $this->app->view->fetch('listado-facturas-fecha.twig', ['docs' => $facturas]);
        $mpdf = new mPDF('c', 'A4', 0, 'Arial', 5, 5, 5, 5, 5, 5);
        $mpdf->WriteHTML($html);
        $nombreFichero = 'ficherostemp3duadjfa/'.'listado-facturas-fecha.pdf';
        $mpdf->Output($nombreFichero, 'F');
        $respuesta = array('link' => $nombreFichero,'datos'=>$facturas);
        return $response->withJson($respuesta);
    }

    public function leer(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $id = ($datos['id']) ? $datos['id'] : 0;
        $factura = Facturas::where('id',$id)->first();

        $data = DB::table('lineas_venta')
            ->leftJoin('albaranes', 'albaranes.id', '=', 'lineas_venta.albaran_id')
            ->select('lineas_venta.*','albaranes.doc as albaran_doc','albaranes.fecha as albaran_fecha')
            ->where('lineas_venta.factura_id','=',$id) ;
        $data->orderBy('albaran_doc');
        $lineas = $data->get();

        $factura['totales'] = [];
        $factura['lineas'] = $lineas;
        $factura['cobros'] = [];
        $factura['depositos'] = [];

        $respuesta = array('resultado' => 'ok', 'datos'=>$factura);

        return $response->withJson($respuesta);
    }

    public function leerLineasVenta(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $id = ($datos['id']) ? $datos['id'] : 0;
        // $lineasAlbaran = LineasVenta::where('factura_id',$id)->get()->toArray();

        $data = DB::table('lineas_venta')
            ->leftJoin('albaranes', 'albaranes.id', '=', 'lineas_venta.albaran_id')
            ->select('lineas_venta.*','albaranes.doc as albaran_doc','albaranes.fecha as albaran_fecha')
            ->where('lineas_venta.factura_id','=',$id) ;
        $data->orderBy('albaran_doc');
        $lineasAlbaran = $data->get();

        $respuesta = array('resultado' => 'ok', 'datos'=>$lineasAlbaran);
        return $response->withJson($respuesta);
    }


    /*
     * Procedimiento para meter facturas directamente sin pasar por el mantenimiento de albaranes.
     * O la hacemos directa y o hacemos una temporal y luego asignamos el número definitivo pero
     * tenemos que comprobar la fecha para que cumpla con la legalidad.
     */
    public function nuevo(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $cliente_id = ($datos['cliente_id']) ? $datos['cliente_id'] : 0;
        $total = ($datos['total']) ? $datos['total'] : 0;

        // numero temporal de factura.
        $numTemp = $this->numeroTempFac();

        $datos = [
            'doc'=>$numTemp,
            'fecha'=>date("Y-m-d"),
            'cliente_id' => $cliente_id,
            'total' => $total,
            'cobrado' => 0,
            'pendiente'=>$total
        ];

        try{
            $registro = new Facturas;
            $registro->saveData($datos);
            $resultado = $registro->id;
        } catch (\Exception $e) {
            $resultado = 0 ;
            $mensajes = $e->getMessage() ;
        }
        // leo el albarán y

        $documento = Facturas::where('id',$registro->id)->first()->toArray();

        $documento['totales'] = [];
        $documento['lineas'] = [];
        $documento['cobros'] = [];
        $documento['depositos'] = [];

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok', 'datos'=>$documento);
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }

    public function cobros(Request $request, Response $response, $args)
    {
        $parametros = $request->getParsedBody();
        $id = $parametros['id'];
        $data = DB::table('cobros')
            ->leftJoin('formaspago', 'formaspago.id', '=', 'cobros.formapago_id')
            ->select('cobros.*','formaspago.descripcion','formaspago.tipo')
            ->where('cobros.factura_id','=',$id) ;
        $data->orderBy('cobros.fecha');
        $datos = $data->get();
        $respuesta = array('datos'=>$datos);
        return $response->withJson($respuesta);
    }
    public function depositos(Request $request, Response $response, $args)
    {
        $parametros = $request->getParsedBody();
        $id = $parametros['id'];
        $data = DB::table('depositos_asignaciones as d')
            ->select('d.*')
            ->where('d.factura_id','=',$id) ;
        $data->orderBy('d.fecha');
        $datos = $data->get();
        $respuesta = array('datos'=>$datos);
        return $response->withJson($respuesta);
    }

    public function numeroFactura($ano,$serie) {
        // comprobamos que existe el contador
        $record = Contadores::where('doc', 'FAC-'.$ano.'-'.$serie)->first();
        // si no existe debemos de crearlo.
        if (!$record) {
            $contador = new Contadores();
            $datos = ['doc'=>'FAC-'.$ano.'-'.$serie, 'valor'=>1];
            $contador->saveData($datos);
        }
        // todoevo hacerlo multiempresa
        try {
            $record = Contadores::where('doc', 'FAC-'.$ano.'-'.$serie)->lockForUpdate()->first();
            $nuevoDoc = $record->valor;
            $record->valor = $nuevoDoc + 1 ;
            $record->save();
            $nuevaFac = 'FAC-'.$ano.'-'.$serie.'-'.str_pad($nuevoDoc, 6, "0", STR_PAD_LEFT);
        } catch (\Exception $e) {
            $nuevaFac = '' ;
        }
        return $nuevaFac ;
    }


    public function verUltima(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $ano = ($datos['ano']) ? $datos['ano'] : 0;
        $serie = ($datos['serie']) ? $datos['serie'] : '01';
        $data = DB::table('facturas')
            ->leftJoin('series', 'series.id', '=', 'facturas.serie_id')
            ->select('facturas.fecha','series.codigo as serie_factura')
            ->whereYear('facturas.fecha', '=', $ano)
            ->where('series.codigo','=',$serie)
            ->orderBy('facturas.fecha','desc');
        $ultimaFactura = $data->first();
        $respuesta = array('datos-ultima-factura'=>$ultimaFactura);
        return $response->withJson($respuesta);
    }


    public function imprimir(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $id = ($datos['id']) ? $datos['id'] : 0;
        $doc = Facturas::where('id',$id)->first()->toArray();
        //$lineasVenta = LineasVenta::where('factura_id',$id)->get()->toArray();

        $data = DB::table('lineas_venta')
            ->leftJoin('albaranes', 'albaranes.id', '=', 'lineas_venta.albaran_id')
            ->select('lineas_venta.*','albaranes.doc as albaran_doc','albaranes.fecha as albaran_fecha')
            ->where('lineas_venta.factura_id','=',$id) ;
        $data->orderBy('albaran_doc');


        // lo convertimos en array todito todo
        $lineasVenta  =json_decode(json_encode($data->get()),true);

        $cliente = Clientes::where('id',$doc['cliente_id'])->get()->toArray();
        $doc['totales'] = [];
        $doc['lineas'] = $lineasVenta;
        $doc['cliente'] = $cliente[0];

        $totalFactura = 0;
        $impuestos = array();
        $datosImpuesto = array();


        // Procedimiento para calcular las bases del iva incluido en los albaranes.
        if ($lineasVenta) {
            foreach ($lineasVenta as $linea) {
                $porcentage = $linea['tipo_iva'];
                if (!isset($impuestos[$porcentage])) {
                    $impuestos[$porcentage] = ['total_iva' => $porcentage, 'base'=> 0,'total'=>0,'tipo_iva'=>$porcentage];
                }
                $impuestos[$porcentage]['total'] = $impuestos[$porcentage]['total'] + $linea['total'];
            };
        }
        foreach ($impuestos as $impuesto) {
            $base = $impuesto['total'] / ( 1 + ($impuesto['tipo_iva'] / 100) ) ;
            $base = round($base,2);
            $impuestos[$porcentage]['total_iva'] = $impuesto['total'] - $base ;
            $impuestos[$porcentage]['base'] = $base;
        };
        $doc['impuestos'] = $impuestos;
        // renderizamos el html
        $html = $this->view->fetch('factura.twig', ['doc' => $doc]);

        $mpdf = new mPDF('c', 'A4', 0, 'Arial', 5, 5, 5, 5, 5, 5);

        $mpdf->WriteHTML($html);
        $nombreFichero = 'ficherostemp3duadjfa/'.$doc['doc'].'.pdf';
        $mpdf->Output($nombreFichero, 'F');
        $respuesta = array('link' => $nombreFichero,'lineasventa'=>$lineasVenta);
        return $response->withJson($respuesta);
    }

    public function anaAlbaranesFactura(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $albaranes  = $datos['albaranes'];
        $factura_id = $datos['factura_id'];

        $albaranesFac = [];
        $factura = [];


        if (count($albaranes) > 0) {
            // creo factura con el cliente y la fecha qu eme mandan

            $factura = Facturas::where('id',$factura_id)->first();
            // actualizamos los albaranes y las lineas de venta.

            if ($factura_id > 0) {
                // actualizo la factura
                Albaranes::whereIn('id', $albaranes)->update(['factura_id' => $factura_id]);
                LineasVenta::whereIn('albaran_id', $albaranes)->update(['factura_id' => $factura_id]);
                Cobros::whereIn('albaran_id', $albaranes)->update(['factura_id' => $factura_id]);

                // recalculamos el total de la factura
                $totales = Facturas::recalcular($factura_id);
                Facturas::where('id', $factura_id)->update(['total'=>$totales['total']]);
                $factura = Facturas::where('id', $factura_id)->get();
            }
        }

        // recalculamos la factura
        // leemos la factura.
        // $albaranesFac = Albaranes::whereIn('id', $albaranes)->get();
        $respuesta = ['estado'=>'ok','datos'=>$albaranesFac,'factura'=>$factura,'id'=>$factura_id];
        return $response->withJson($respuesta);
    }

    public function quitarAlbaranesFactura(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $albaranes  = $datos['albaranes'];
        $factura_id = $datos['factura_id'];

        $albaranesFac = [];
        $factura = [];


        if (count($albaranes) > 0) {
            // creo factura con el cliente y la fecha qu eme mandan

            $factura = Facturas::where('id',$factura_id)->first();
            // actualizamos los albaranes y las lineas de venta.

            if ($factura_id > 0) {
                // actualizo la factura
                Albaranes::whereIn('id', $albaranes)->update(['factura_id' => 0]);
                LineasVenta::whereIn('albaran_id', $albaranes)->update(['factura_id' =>0]);
                Cobros::whereIn('albaran_id', $albaranes)->update(['factura_id' => 0]);

                // recalculamos el total de la factura
                $totales = Facturas::recalcular($factura_id);
                Facturas::where('id', $factura_id)->update(['total'=>$totales['total']]);
                $factura = Facturas::where('id', $factura_id)->get();
            }
        }

        // recalculamos la factura
        // leemos la factura.
        // $albaranesFac = Albaranes::whereIn('id', $albaranes)->get();
        $respuesta = ['estado'=>'ok','datos'=>$albaranesFac,'factura'=>$factura,'id'=>$factura_id];
        return $response->withJson($respuesta);
    }
}