<?php

namespace App\Controllers;

use Slim\Views\Twig;
use Slim\Http\Request;
use Slim\Http\Response ;

use Illuminate\Database\Capsule\Manager as DB;
use Productos ;
use Familias ;
use Contadores ;
use mPDF ;
use \App\BusinessProcess\ProductosBP as ProductosBP;

class ProductosController
{
    private $app;
    private $view;
    private $utiles ;

    public function __construct($app, Twig $view)
    {
        $this->app = $app;
        $this->view = $view;
        $container = $this->app->getContainer();
        $this->utiles = $container->get('Utiles') ;
    }
    public function index()
    {
        echo ':)';
    }

    public function lista(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $descripcion = $datos['descripcion'];
        $id = $datos['id'];
        $data = DB::table('productos');
        $data->select('productos.*',DB::raw('(productos.precio_compra + (productos.precio_compra * (21 / 100) )) as precio_compra_iva') );
        if ($descripcion) {
            $data->where('productos.descripcion', 'like', "%" . strtoupper($descripcion) . "%");
            $data->orWhere('productos.producto', '=', "" . strtoupper($descripcion) . "");
            $data->orWhere('productos.codigo_barras', '=', "" . strtoupper($descripcion) . "");
        }

        if($id > 0) {
            $data->where('productos.id','=',$id);
        }

        $data->orderBy('productos.descripcion');
        $datosProductos = $data->get();
        $array = array('resultado' => 'ok','datos' => $datosProductos);

        return $response->withJson($array);
}

    public function listaGolf(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $botontpv_id = 0 ;
        if (isset($datos['id'])) $botontpv_id = $datos['id'];
        $data = DB::table('productos');
        $data->select('productos.*',DB::raw('(productos.precio_compra + (productos.precio_compra * (21 / 100) )) as precio_compra_iva') );
        if ($botontpv_id > 0) {
            $data->where('productos.botontpv_id', $botontpv_id);
            $data->orderBy('productos.ordenbotontpv');
        } else {
            $data->where('productos.favorito', 1);
            $data->orderBy('productos.ordenbotontpv');
        }
        // $data->whereIn('productos.grupo_id',array(2,3));
        $data->orderBy('productos.descripcion');
        $datosProductos = $data->get();
        $array = array('resultado' => 'ok','datos' => $datosProductos);

        return $response->withJson($array);
    }

    public function listaFamilias(Request $request, Response $response, $args)
    {
        $parametros = $request->getParsedBody();
        $descripcion = $parametros['descripcion'];
        $id = $parametros['id'];
        $data = DB::table('familias');
        if ($descripcion) {
            $data->where('familias.descripcion', 'like', "" . strtoupper($descripcion) . "%");
            $data->orWhere('familias.producto', '=', "" . strtoupper($descripcion) . "");
        }
        if($id > 0) {
            $data->where('familias.id','=',$id);
        }
        $data->orderBy('familias.descripcion');
        $datos = $data->get();
        $array = array('resultado' => 'ok','datos' => $datos);

        return $response->withJson($array);
    }


    public function ver(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $id = $datos['id'];
        $data = DB::table('productos')->where('id', '=', $id)->get();
        $array = array('resultado' => 'ok','datos' => $data);

        return $response->withJson($array);
    }

    public function verDatosProducto(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $id = $datos['id'];
        $data = DB::table('productos')->where('id', '=', $id)->get();
        $ultimaCompra = DB::table('lineas_compra as l')
            ->leftJoin('facturas_compra as f','l.factura_id','=','f.id')
            ->select('l.*','f.fecha')
            ->where('l.producto_id','=',$id)->orderBy('f.fecha','DESC')->first();


        $array = array('resultado' => 'ok','datos' => $data,'ultima_compra'=>$ultimaCompra );

        return $response->withJson($array);
    }

    public function grabar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $datos['precio_venta'] = round($datos['precio_venta'],2);
        $mensajes = array();

        if ($datos['id'] == 0) {
            $datos['producto'] = $this->generaCodigo();
        }

        $datos['codigo_barras'] = $this->generateEAN( $datos['producto']);

        $registro = $this->utiles->grabaModelo('Productos',$datos);

        if ($datos['producto'] == '') {
            $id = $registro['id']; // id del producto grabado
            Productos::where('id', $id)->update(['producto' => str_pad($id, 8, "0", STR_PAD_LEFT)]);
        }

        $resultado = $registro['resultado'];
        $errores =  json_decode($registro['mensajes']);
        if ($resultado == 'ok') {
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko','errores'=>$errores);
        }

        return $response->withJson($respuesta);
    }

    public function generateEAN($number)
    {
        $code = '200' . str_pad($number, 9, '0');
        $weightflag = true;
        $sum = 0;
        // Weight for a digit in the checksum is 3, 1, 3.. starting from the last digit.
        // loop backwards to make the loop length-agnostic. The same basic functionality
        // will work for codes of different lengths.
        for ($i = strlen($code) - 1; $i >= 0; $i--)
        {
            $sum += (int)$code[$i] * ($weightflag?3:1);
            $weightflag = !$weightflag;
        }
        $code .= (10 - ($sum % 10)) % 10;
        return $code;
    }

    public function generaCodigo() {
        $nuevoProducto = 0;

        // comprobamos que existe el contador
        $record = Contadores::where('doc', 'PRODUCTO')->first();

        // si no existe debemos de crearlo.
        if (empty($record)) {
            $contador = new Contadores();
            $datos = ['doc'=>'PRODUCTO', 'valor'=>1];
            $contador->saveData($datos);
        }
        // todoevo hacerlo multiempresa
        try {
            $record = Contadores::where('doc', 'PRODUCTO')->lockForUpdate()->first();
            $nuevoProducto = $record->valor;
            $record->valor = $nuevoProducto + 1 ;
            $record->save();
            $nuevoProducto = str_pad($nuevoProducto, 8, "0", STR_PAD_LEFT);
        } catch (\Exception $e) {
            $nuevoProducto = 0 ;
        }
        return $nuevoProducto ;
    }

    public function actualizaStock(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $id = $datos['id'];
        $stockInv = $datos['stock_inv'];
        $stockAct = $datos['stock_act'];

        $inventario = array();
        $inventario['id'] = 0 ;
        $inventario['fecha']         = date("Y-m-d H:i:s");
        $inventario['producto_id']   = $id ;
        $inventario['empresa_id']    = 1;
        $inventario['almacen_id']    = 1;
        $inventario['cantidad']    = $stockInv;
        $inventario['stock_actual']    = $stockAct;

        $registro = $this->utiles->grabaModelo('Inventario',$inventario);

        try{
            Productos::where('id', $id)->update(['stock' => $stockInv]);
            $resultado = 1 ;
        } catch (\Exception $e) {
            $resultado = 0 ;
        }

        $prodBP = new ProductosBP();
        $resultadoActStock = $prodBP->actStockProducto($id);

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok','act_stock'=>$resultadoActStock);
        } else {
            $respuesta = array('resultado' => 'ko');
        }

        return $response->withJson($respuesta);

    }
    public function actualizaPrecio(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $id = $datos['id'];
        $precioVenta = $datos['precio_venta'];
        try{
            Productos::where('id', $id)->update(['precio_venta' => $precioVenta]);
            $resultado = 1 ;
        } catch (\Exception $e) {
            $resultado = 0 ;
        }

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko');
        }

        return $response->withJson($respuesta);

    }

    public function etiquetas(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $facturasHtml = '' ;
        $productos = $datos['datos'];
        $fila = $datos['fila'];
        $columna =$datos['columna'];

        $empiezaEn = (($fila-1)*6) + $columna ;

        $articulosEtiquetas = array();


        // bucle para añadir las vacías.
        for ($i=1;$i<$empiezaEn;$i++) {
            $articulosEtiquetas[] = array('producto'=>'','descripcion'=>'','imprime'=>false);
        }
        // Añadimos los productos
        foreach( $productos as $prod ) {
            $numeroEtiquetas = $prod['etiquetas'];
            for ($i=1;$i<=$numeroEtiquetas;$i++) {
                $articulosEtiquetas[] = array('producto' => $prod['producto'],
                    'descripcion' => $prod['descripcion'],
                    'precio' => $prod['precio_venta'],
                    'codigo_barras' => $prod['codigo_barras'],
                    'imprime' => true);
            }
        }

        $html = $this->view->fetch('etiquetas.twig', ['etiquetas' => $articulosEtiquetas]);
        $mpdf = new mPDF('c', 'A4', 0, 'Arial', 5, 5, 5, 5, 5, 5);
        $mpdf->WriteHTML($html);
        $nombreFichero = 'ficherostemp3duadjfa/'.'etiquetas.pdf';
        $mpdf->Output($nombreFichero, 'F');
        $respuesta = array('link' => $nombreFichero);
        return $response->withJson($respuesta);
        
    }
    public function etiquetasCompras(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $facturasHtml = '' ;
        $productos = $datos['datos'];

        $fila = $datos['fila'];
        $columna =$datos['columna'];

        $empiezaEn = (($fila-1)*6) + $columna ;

        $articulosEtiquetas = array();


        // bucle para añadir las vacías.
        for ($i=1;$i<$empiezaEn;$i++) {
            $articulosEtiquetas[] = array('producto'=>'','descripcion'=>'','imprime'=>false);
        }
        // Añadimos los productos
        foreach( $productos as $prod ) {
            $numeroEtiquetas = $prod['etiquetas'];
            $producto = DB::table('productos')->where('id', '=', $prod['producto_id'])->get();
            if (isset($producto)) {
                for ($i = 1; $i <= $numeroEtiquetas; $i++) {
                    $articulosEtiquetas[] = array('producto' => $producto[0]->producto,
                        'descripcion' => $producto[0]->descripcion,
                        'precio' => $producto[0]->precio_venta,
                        'codigo_barras' => $producto[0]->codigo_barras,
                        'imprime' => true);
                }

            }
        }

        $html = $this->view->fetch('etiquetas.twig', ['etiquetas' => $articulosEtiquetas]);
        $mpdf = new mPDF('c', 'A4', 0, 'Arial', 5, 5, 5, 5, 5, 5);
        $mpdf->WriteHTML($html);
        $nombreFichero = 'ficherostemp3duadjfa/'.'etiquetas.pdf';
        $mpdf->Output($nombreFichero, 'F');
        $respuesta = array('link' => $nombreFichero);
        return $response->withJson($respuesta);

    }

    public function recalculoStock(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $producto_id = $datos['producto_id'];
        $prodBP = new ProductosBP();
        $respuesta = array( 'datos' => $prodBP->actStockProducto($producto_id));
        return $response->withJson($respuesta);
    }

    public function recalculoStocks(Request $request, Response $response, $args)
    {
        $prodBP = new ProductosBP();

        $productos = DB::table('productos')->get();
        foreach($productos as $producto) {
            $prodBP->actStockProducto($producto->id);
        }

        $respuesta = array('estado'=>'ok');
        return $response->withJson($respuesta);
    }

    public function verIntenvarioFecha(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $fecha = $datos['fecha'];

        $prodBP = new ProductosBP();
        $inventario = [];

        $productos = DB::table('productos')->get();
        foreach($productos as $producto) {
            $infoStock = $prodBP->verStockFecha($producto->id,$fecha);
            $productoInventario = (array) $producto;
            $productoInventario['info_stock'] = $infoStock ;
            $inventario[] = $productoInventario ;

        }

        $resultado = array('estado'=>'ok','datos'=>$inventario);
        return $response->withJson($resultado);
    }

    public function extracto(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $producto_id = $datos['producto_id'];
        $prodBP = new ProductosBP();
        $resultado = array( 'datos' => $prodBP->extracto($producto_id));


        return $response->withJson($resultado);
    }

    public function borrar(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $id = $datos['id'];
        $prodBP = new ProductosBP();
        $tieneMovimientos = $prodBP->comprobarProducto($id);

        if ($tieneMovimientos == 0) {
            $data = DB::table('productos')->where('id', '=', $id)->delete();
            $respuesta = array('resultado' => 'ok');
        } else {
            $respuesta = array('resultado' => 'ko','tienemovimientos'=>$tieneMovimientos);
        }

        return $response->withJson($respuesta);

    }


}