<?php

namespace App\Controllers;

use Dreservas;
use Greenfees;
use Greenfeestemp;
use Illuminate\Database\Capsule\Manager as DB;
use PDO;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Reservas;
use Slim\Views\Twig;

class ReservasController
{

    private $app;
    private $view;
    private $utiles;
    private $pdo;

    public function __construct($app, Twig $view)
    {
        $this->app = $app;
        $this->view = $view;
        $container = $this->app->getContainer();
        $this->utiles = $container->get('Utiles');
        $this->pdo = $container->get('Pdo');
    }


    public function index()
    {
        echo ':)';
    }

    public function leerGreenfees(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $idreservahora = $datos['idreservahora'];
        $idReserva = $datos['idreserva'];
        $reserva = Reservas::where('idreserva', '=', $idReserva)->first();
        $dreservas = Dreservas::where('idreserva', '=', $idReserva)->orderBy('fecha', 'ASC')->get();

        $greenfees = DB::table('greenfees as gf')
            ->leftJoin('tickets as ti', 'gf.ticket_id', '=', 'ti.id')
            ->leftJoin('albaranes as alb', 'gf.albaran_id', '=', 'alb.id')
            ->leftJoin('clientes as c', 'c.id', '=', 'gf.cliente_id')
            ->leftJoin('abonos as ab', 'ab.id', '=', 'gf.abono_id')
            ->leftJoin('tarifas_golf as tg', 'tg.id', '=', 'gf.tarifa_id')
            ->select('gf.*', 'ti.doc', 'ab.abono', 'tg.tarifa', 'alb.doc as albaran_doc')
            ->where('gf.idreservahora', '=', $idreservahora)
            ->orderBy('gf.fecha', 'asc')
            ->get();

        $respuesta = ['estado' => 'ok', 'greenfees' => $greenfees, 'reserva' => $reserva, 'dreservas' => $dreservas];
        return $response->withJson($respuesta, 200, JSON_NUMERIC_CHECK);
    }

    public function ocupacionMensual(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();
        $fecha = $datos['fecha'];

        $mes = substr($fecha, 5, 2);
        $ano = substr($fecha, 0, 4);

        $data = DB::table('dreservas as d')->selectRaw('d.fecha,month(d.fecha) as mes,year(d.fecha) as ano,day(d.fecha) as dia, sum(d.jugadores) as total_greenfees,sum(d.buggies) as total_buggies')
            ->whereMonth('d.fecha', '=', $mes)
            ->whereYear('d.fecha', '=', $ano)
            ->groupBy('d.fecha')->orderBy('d.fecha');

        $registros = $data->get();
        $respuesta = array('resultado' => 'ok', 'datos' => $registros);


        return $response->withJson($respuesta);
    }

    public function reservaNueva(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $datos = toArray($datos);

        $fechaReserva = $datos['fecha'];
        $recorrido = $datos['recorrido'];
        $numeroJugadores = $datos['jugadores'];
        $numeroBuggies = $datos['buggies'];
        $fly = $datos['fly'];

        $cliente_id = 0;
        if (array_key_exists('cliente_id', $datos)) {
            $cliente_id = $datos['cliente_id'];
        }

        // los datos de los greenfees vienen en las reservas de los socios / abonados.
        // añadimos a los greenfees temporales que generamos, los datos que nos traemos en $datosGreenfees.
        // El número de datosgreenfes y los greenfees temporales son los mismos.

        /*
         * Hay que traer el código del aobno y el cliente.
         * Así como la tarifa
         */


        /*
         * Por eso es aquí donde tenemos que comprobar el saldo delos abonos que vienen. Primero agrupamos por abono y luego comprobamos
         * cada uno de los abonos y su saldo.
         */

        $datosGreenfees = $datos['datos_greenfees'];

        $idReservaHora = $this->utiles->generaIdReserva();

        // todoevo ver si esto lo podemos hacer para utilizar al añadir nuevas dreservas a la reserva principal en la
        // edición de la reserva.
        $idReserva = $this->utiles->generaIdReserva();

        /*
         * Si tenemos un id de hora de reserva es que estamos modificando la reserva y por lo tanto no debemos de añadir más greenfees.
         */
        /* if ($idReservaHora) {
            Greenfeestemp::where('idreservahora','=',$idReservaHora)->delete();
            Greenfees::where('idreservahora','=',$idReservaHora)->update(array('modificando' => 1));
        } */
        // todoevo cambio sql
        // ALTER TABLE  `greenfees` ADD  `modificando` INT NULL DEFAULT  '0' AFTER  `idreservahora` ;
        // marcamos el fly inicial en el que se empieza a reservar. Esto nos sirve para poder coger
        // bien el precio del greenfee y los buggies. le restamos 1

        $flyInicial = $fly;
        $flyInicialdb = $fly - 1;
        $data = array();
        $timer_start = microtime(TRUE);
        // genero la tabla temporal
        $mysqltemp = <<<SQL
        CREATE TEMPORARY TABLE horasgftemp (
        recorrido integer null default 0,
        `hora` time NOT NULL,
        `fly` int(11) NOT NULL,
        `bloqueada` int(11) DEFAULT NULL,
        `fecha` date NOT NULL
        ) ENGINE=InnoDB  DEFAULT CHARSET=utf8 AUTO_INCREMENT=1
SQL;

        $stmt = $this->pdo->prepare($mysqltemp);

        $stmt->execute();


        $intervalo = 8;

        $hora = strtotime('07:30');
        for ($i = 1; $i < 90; $i++) {
            $horas = date('H:i', $hora);
            $intervalo = 8;

            $mysqltemp = "INSERT INTO horasgftemp (fecha,recorrido,hora,fly) values (STR_TO_DATE(:fechareserva,'%Y-%m-%d'),1,:hora,:fly)";
            $stmt = $this->pdo->prepare($mysqltemp);
            $stmt->bindParam(":fechareserva", $fechaReserva);
            $stmt->bindParam(":hora", $horas);
            $stmt->bindParam(":fly", $i);
            $stmt->execute();

            $mysqltemp = "INSERT INTO horasgftemp (fecha,recorrido,hora,fly) values (STR_TO_DATE(:fechareserva,'%Y-%m-%d'),2,:hora,:fly)";
            $stmt = $this->pdo->prepare($mysqltemp);
            $stmt->bindParam(":fechareserva", $fechaReserva);
            $stmt->bindParam(":hora", $horas);
            $stmt->bindParam(":fly", $i);
            $stmt->execute();

            /*  $mysqltemp = "INSERT INTO horasgftemp (fecha,recorrido,hora,fly) values (STR_TO_DATE(:fechareserva,'%Y-%m-%d'),3,:hora,:fly)";
              $stmt =  $this->pdo->prepare($mysqltemp);
              $stmt->bindParam(":fechareserva", $fechaReserva);
              $stmt->bindParam(":hora", $horas);
              $stmt->bindParam(":fly", $i);
              $stmt->execute();*/

            $hora = strtotime($intervalo . ' minutes', $hora);

        }

        // relleno la tabla temporal con las reservas que hay en el día


        // sacamos las reservas del día
        // ojo, todavía no tiene el recorrido

// genero bloqueos y veo los jugadores que tengo
        $sql = <<<SQL
  SELECT horasgftemp.fecha, horasgftemp.hora, horasgftemp.recorrido,horasgftemp.fly,
   (COUNT(DISTINCT greenfees.id) + COUNT(DISTINCT greenfeestemp.id)) as jugadores,
      bloqueos.id as bloqueo
  FROM horasgftemp
  LEFT JOIN greenfees
    ON horasgftemp.fecha = greenfees.fecha
        and  horasgftemp.hora = greenfees.hora
                and  horasgftemp.recorrido = greenfees.recorrido and greenfees.modificando <> 1
            LEFT JOIN greenfeestemp
            ON (horasgftemp.fecha = greenfeestemp.fecha
              and  horasgftemp.hora = greenfeestemp.hora
              and  horasgftemp.recorrido = greenfeestemp.recorrido
              and  greenfeestemp.idreserva = "$idReserva")
            LEFT JOIN bloqueos
            on (horasgftemp.fecha = bloqueos.fechaini ) AND
            (horasgftemp.hora between bloqueos.horaini and bloqueos.horafin) and
            (horasgftemp.recorrido = bloqueos.recorrido)
            where horasgftemp.fecha = "$fechaReserva" and horasgftemp.recorrido = $recorrido
            GROUP BY horasgftemp.fecha,horasgftemp.recorrido,horasgftemp.fly
SQL;

        $stmt = $this->pdo->prepare($sql);
        $stmt->execute();
        $reservas = $stmt->fetchAll(PDO::FETCH_ASSOC);

        // falta precio de los greenfees

        $greenFeesReservar = $numeroJugadores;
        $reservaNoValida = false;

        $time = 0;
        $salir = 0;

        $flydb = $fly - 1;


        $jugadoresLibres = 0;
        // vemos que fly tiene primero greenfees libres esto es para el frontend

        // si el fly es mayor que 80 entonces la reserva no es válida.
        if ($fly > 90) {
            $reservaNoValida = true;
            $greenFeesReservar = 0;
        }

        // comprorbar si hay tarifa de precios activa
        // Nos traemos el precio dela tarifa

        $horaTarifa = $reservas[$flydb]['hora'];


        //todoevo cambiar para que se vea la hoja de la tarifa
        // todoevo quitar, todo lo traeremos de la funcion ver preciooferta
        //  $precios = verPrecioGF($fechaReserva, $horaTarifa);
        //  $precioGreenfee = $precios[0]['preciogf'];
        //  $precioBuggie = $precios[0]['preciobuggie'];

        // todoevo, ver los precios de oferta

        // esto es superimportante. El flyinicial db es cuando empieza la reserva y es lo que mandamos con el presupuesto.

        $flyInicialdb = 0;

        while ($greenFeesReservar > 0) {
            // tenemos que comprobar si existe el fly
            $horaReserva = $reservas[$flydb]['hora'];
            // Hecho Comprobamos que si la fecha es la actual y la hora es menor que la actual, no hay reserva que se peuda hacer
            // esto si debe de estar habilitado para admin.
            // debemos de pasarle un flag admin para que esta limitación no se pueda tener en cuenta.
            /*  if (!$admin) {
                 if ( $fechaReserva < date('Y-m-d') ) {
                     $reservaNoValida = true;
                     $greenFeesReservar = 0;
                 }

                 if ( $fechaReserva == date('Y-m-d') && $horaReserva < date('H:i') ) {
                     $reservaNoValida = true;
                     $greenFeesReservar = 0;
                 }
             } */


            // hacemos una comprobación para que no puedan reservar con llamadas
            if ($horaReserva > '19:44') {
                $reservaNoValida = true;
                $greenFeesReservar = 0;
            }
            if ($reservaNoValida) {
                $bloqueo = 99999;
                $greenFeesLibresHora = 0;
            } else {
                $bloqueo = $reservas[$flydb]['bloqueo'];
                $greenFeesLibresHora = 4 - $reservas[$flydb]['jugadores'];
            }
            //
            // si en esa hora hay greenfees libres
            if ($greenFeesLibresHora > 0 && $bloqueo < 1) {
                // si lo que nos queda por resrervar es mayor o igual que lo libre
                if ($greenFeesLibresHora >= $greenFeesReservar) {
                    if ($flyInicialdb == 0) $flyInicialdb = $fly;
                    for ($i = 1; $i < $greenFeesReservar + 1; $i++) {
                        $data[] = array(
                            'fecha' => $fechaReserva,
                            'hora' => $horaReserva,
                            'fly' => $fly,
                            'recorrido' => $recorrido,
                            'jugador' => 1,
                            'idreserva' => $idReserva,
                            'idreservahora' => $idReservaHora,
                            'tarifa_id'=>0,
                            'buggie' => 0);
                    }

                    // $totalFly = $precioGreenfee * $greenFeesReservar;
                    // $total = $total + $totalFly ;
                    $greenFeesReservar = 0;
                    $totalFly = 0;
                } // si no , reservamos loq ue queda libre y pasamos a la siguiente hora
                else {
                    if ($flyInicialdb == 0) $flyInicialdb = $fly;
                    for ($i = 1; $i < $greenFeesLibresHora + 1; $i++) {
                        $data[] = array(
                            'fecha' => $fechaReserva,
                            'hora' => $horaReserva,
                            'fly' => $fly,
                            'recorrido' => $recorrido,
                            'jugador' => 1,
                            'idreserva' => $idReserva,
                            'idreservahora' => $idReservaHora,
                            'tarifa_id'=>0,
                            'buggie' => 0
                        );
                    }
                    $greenFeesReservar = $greenFeesReservar - $greenFeesLibresHora;
                    //  quitar, todo lo traeremos de la funcion ver preciooferta
                    // $totalFly = $precioGreenfee * $greenFeesLibresHora;
                    // $total = $total + $totalFly ;
                    $fly++;
                    $flydb++;
                }
            } else {
                $totalFly = 0;
                $fly++;
                $flydb++;
            }
        }

        $estado = 'ok';
        $totalPresu = array(
            'ofertasAplicadas' => 0,
            'jugadores' => $numeroJugadores,
            'totaljugadores' => 0,
            'preciogreenfee' => 0,
            'buggies' => $numeroBuggies,
            'preciobuggie' => 0,
            'totalbuggies' => 0,
            'totaloferta' => 0,
            'ofertasposibles' => 0,
            'totalreserva' => 0,
            'concepto' => ''
        );
        if ($reservaNoValida) {
            $estado = 'ko';
            $reserva = array();
            $data = array();
        } else {


            // todoevo aquí tenemos que buscar la oferta
            //$totalPresu = $this->utiles->verPrecioOferta($fechaReserva,$horaTarifa,$numeroJugadores,$numeroBuggies);

            $totalPresu['preciobuggie'] = 0;
            $totalPresu['preciogreenfee'] = 0;

            foreach ($datosGreenfees as $dgf) {
                $totalPresu['totalreserva'] = $totalPresu['totalreserva'] + $dgf['preciogf'];
            }


            $dreserva[] = array(
                'fecha' => $fechaReserva,
                'hora' => $reservas[$flyInicialdb - 1]['hora'], // cuando mandamos la hora, el flyinicialdb es el index de la tabla de reservas
                'fly' => $flyInicialdb, // pero el fly es el flydbinicial + 1,
                'recorrido' => $recorrido,
                'jugadores' => $numeroJugadores,
                'precio' => $totalPresu['preciogreenfee'],

                'buggies' => $numeroBuggies,
                'preciobuggies' => $totalPresu['preciobuggie'], // esto ya no tiene sentido
                'total' => $totalPresu['totalreserva'],
                'idreserva' => $idReserva,
                'idreservahora' => $idReservaHora,
                'notas' => ''
            );

            $reserva = array(
                'fechareserva' => $fechaReserva,
                'total' => $totalPresu['totalreserva'],
                'nombre' => $datos['nombre_jugador'],
                'totaldto' => 0,
                'admin' => 1,
                'idreserva' => $idReserva,
                'formapago' => '',
                'aceptacondiciones' => true,
                'recordardatos' => true,
                'idioma' => 'es',
                'cliente_id' => $cliente_id

            );
            $this->utiles->grabaGFTemp($data, $this->pdo);
        }


        /*

        todoevo aquí tenemos que aplicar las reglas de negocio para cambiar el total de la reserva si es necesario

        */

        // Aquí comienza la grabación de la reserva

        // $reserva = $datos['reserva'];
        //unset($reserva['id']); // quito el id del array para que no me de problemas

        $dreservadef = array();
        $totalReserva = 0;

        $errorReserva = 0;

        // todoevo esto creo qu epuede venir de arriba
        // todoevo muy importante
        /* $horasGreenfees = DB::table('greenfeestemp as d')
            ->selectRaw('d.fecha, d.fly, d.recorrido, d.idreservahora,d.hora,COUNT(*) AS greenfees')
            ->where('idreserva',$idReserva)
            ->groupBy('d.fecha,d.fly,d.recorrido')->get();
        $mensaje = '' ;

        foreach($horasGreenfees as $dre) {
            $condicion = array('fecha' => $dre->fecha, 'recorrido' => $dre->recorrido,'fly'=>$dre->fly);
            $totalGreenfees = Greenfees::where($condicion)->count();
            $greenfeesDisponibles = 4 - $totalGreenfees ;
            if ($greenfeesDisponibles < $dre['greenfees']) {
                $error[]=$dre;
                $errorReserva = 1 ;
            }
        } */


        if ($errorReserva == 1) {
            $mensaje = 'COLISION';
            $respuesta = array('resultado' => 'ko', 'mensaje' => $mensaje);
            return $response->withJson($respuesta);

        } else {

            $reserva['totaldto'] = 0;
            if ($reserva['formapago'] == 'TC' || $reserva['formapago'] == 'PA') {
                $reserva['totaldto'] = $reserva['totalbruto'] * 0.1;
                $reserva['total'] = $reserva['totalbruto'] - $reserva['totaldto'];
            }
            //print_r($reserva);
            //print_r($dreserva);

            // buscamos la reserva
            $reservabd = Reservas::where('idreserva', '=', $reserva['idreserva'])->first();

            // si existe
            if (!is_null($reservabd)) {
                try {
                    $reserva = Reservas::where('idreserva', '=', $reserva['idreserva'])->update($reserva);
                    $resultado = 1;
                } catch (\Exception $e) {
                    $mensaje = $e->getMessage();
                    $resultado = 0;
                }
            } else {
                $datos['fechareserva'] = date("Y-m-d H:i:s");
                try {
                    $reserva = Reservas::create($reserva);
                    $resultado = $reserva->id;
                } catch (\Exception $e) {
                    $mensaje = $e->getMessage();
                    $resultado = 0;
                }
            }

            // Hay que grabar dreserva
            // si ha habido suerte y tenemos un número de id de reserva


            if ($resultado > 0) {
                $greenfeestemp = Greenfeestemp::where('idreserva', '=', $idReserva)->get()->toArray();
                if ($greenfeestemp) {
                    $datosgf = array();
                    $i = 0;
                    foreach ($greenfeestemp as $gf) {
                        unset($gf['id']);
                        // esto nos sirve tanto para la reserva de socios como para la de suecos y abonados.
                        // estos datos viene de 
                        if ($datosGreenfees) {

                            if ($numeroBuggies > 0) {
                                $gf['buggie'] = 1;
                                $numeroBuggies = $numeroBuggies - 1;
                            } else {
                                $gf['buggie'] = 0;
                            }
                            $gf['cliente_id'] = $datosGreenfees[$i]['cliente_id'];
                            $gf['abono_id'] = $datosGreenfees[$i]['abono_id'];
                            $gf['nombre_jugador'] = $datosGreenfees[$i]['nombre_jugador'];
                            $gf['tarifa'] = $datosGreenfees[$i]['tarifa'];
                            $gf['tarifa_id'] = $datosGreenfees[$i]['tarifa_id'];
                            $gf['preciogf'] = $datosGreenfees[$i]['preciogf'];
                            $gf['preciobuggie'] = $datosGreenfees[$i]['preciobuggie'];
                            $gf['hoyos'] = $datosGreenfees[$i]['hoyos'];
                            $gf['essocio'] = $datosGreenfees[$i]['essocio'];
                        }
                        $datosgf[] = $gf;
                        $i++;
                    }
                    $resultadoGF = Greenfees::insert($datosgf);
                    $resultadoDR = Dreservas::insert($dreserva);
                    // todoevo hay que dejar los greenfees modificando a 0
                    $datosgf = Greenfees::where('idreservahora','=',$idReservaHora)->get();
                }
            }


            $respuesta = array();

            if ($resultado > 0) {
                $reserva['greenfees'] = $datosgf;
                $reserva['dreservas'] = $dreserva;
                $respuesta = array('resultado' => 'ok', 'id' => $resultado, 'reserva' => $reserva);
                Greenfeestemp::where('idreserva', '=', $idReserva)->delete();
            } else {
                $respuesta = array('resultado' => 'ko', 'mensaje' => $mensaje);
            }

            return $response->withJson($respuesta);

        }
    }

    public function actualizaGreenfeesReserva(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $tarifa = $datos['tarifa'];
        $greenfees = $datos['greenfees'];
        $idReserva = $datos['idreserva'];
        $cliente = $datos['cliente'];

        foreach ($greenfees as $gf) {
            Greenfees::where('id', $gf['id'])->update(['tarifa_id' => $tarifa['id'], 'preciogf' => $tarifa['pvpgf18'], 'preciobuggie' => $tarifa['pvpbuggies'],'cliente_id'=>$cliente['id']]);
        }
        $greenfeesReserva = DB::table('greenfees as gf')
            ->leftJoin('tickets as ti', 'gf.ticket_id', '=', 'ti.id')
            ->leftJoin('clientes as c', 'c.id', '=', 'gf.cliente_id')
            ->leftJoin('abonos as ab', 'ab.id', '=', 'gf.abono_id')
            ->leftJoin('tarifas_golf as tg', 'tg.id', '=', 'gf.tarifa_id')
            ->select('gf.*', 'ti.doc', 'ab.abono', 'tg.tarifa')
            ->where('gf.idreserva', '=', $idReserva)
            ->orderBy('gf.fecha', 'asc')
            ->get();

        $respuesta = array('resultado' => 'ok', 'greenfees' => $greenfeesReserva);
        return $response->withJson($respuesta);
    }

    public function actualizaGreenfeesReservaHora(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $tarifa = $datos['tarifa'];
        $greenfees = $datos['greenfees'];
        $idReservaHora = $datos['idreservahora'];

        foreach ($greenfees as $gf) {
            Greenfees::where('id', $gf['id'])->update(['tarifa_id' => $tarifa['id'], 'preciogf' => $tarifa['pvpgf18'], 'preciobuggie' => $tarifa['pvpbuggies']]);
        }
        $greenfeesReserva = DB::table('greenfees as gf')
            ->leftJoin('tickets as ti', 'gf.ticket_id', '=', 'ti.id')
            ->leftJoin('clientes as c', 'c.id', '=', 'gf.cliente_id')
            ->leftJoin('abonos as ab', 'ab.id', '=', 'gf.abono_id')
            ->leftJoin('tarifas_golf as tg', 'tg.id', '=', 'gf.tarifa_id')
            ->select('gf.*', 'ti.doc', 'ab.abono', 'tg.tarifa')
            ->where('gf.idreservahora', '=', $idReservaHora)
            ->orderBy('gf.fecha', 'asc')
            ->get();

        $respuesta = array('resultado' => 'ok', 'greenfees' => $greenfeesReserva);
        return $response->withJson($respuesta);
    }
}    