<?php

namespace App\Controllers;

use Slim\Views\Twig;
use Slim\Http\Request;
use Slim\Http\Response ;
use Illuminate\Database\Capsule\Manager as DB;
use Tickets ;
use LineasVenta ;
use Contadores ;
use Cobros ;
use Clientes ;
use Greenfees ;
use \App\BusinessProcess\ProductosBP as ProductosBP;


use mPDF;

class TicketsController
{
    private $app;
    private $view;
    private $utiles ;

    public function __construct($app, Twig $view)
    {
        // parent::__construct();
        $this->app = $app;
        $this->view = $view;
        $container = $this->app->getContainer();
        $this->utiles = $container->get('Utiles') ;
    }
    public function index()
    {
        echo ':)';
    }


    public function lista(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $cliente_id = 0 ;
        $esDiario = false ;
        if (array_key_exists('cliente_id',$datos)) {
            $cliente_id = $datos['cliente_id'] ;
        }

        if (!empty($datos['es_diario'])) {
            $esDiario = true ;
        }

        // Pasamos los datos a un array
        $tickets = new Tickets();
        $data = $tickets->lista($datos['fecha_ini'],$datos['fecha_fin'],'',$cliente_id,$esDiario);
        $respuesta = array('resultado' => 'ok','datos' => $data);



        return $response->withJson($respuesta);
    }

    public function grabar(Request $request, Response $response, $args)
    {

        $parametros = $request->getParsedBody();

        $lineasAnteriores = isset($parametros['lineas_anteriores']) ? $parametros['lineas_anteriores'] : array();

        $datos = $parametros['documento'];
        $tipo = $parametros['tipo'];
        $doc = $datos['doc'] ;

        $lineas = $datos['lineas'];

        unset($datos['lineas']);
        unset($datos['impuestos']);
        unset($datos['cobros']);
        unset($datos['totales']);

        $registro = $this->utiles->grabaModelo('Tickets',$datos);

        $resultado = $registro['id'];
        $mensajes =  $registro['mensajes'];
        
        // si tenemos un id registro
        if ($resultado > 0) {
            LineasVenta::where('ticket_id',$resultado)->delete();
            foreach($lineasAnteriores as $linea) {
                // todoevo actualizar stock añadiendo
            }

            // todoevo control de errores
            if ($tipo != 'A' && $datos['total'] != 0) {
                if (substr($datos['doc'],0,3) == 'TIC') {
                    $registro = Tickets::find((int)$resultado);
                    $doc = $this->numeroTicket();
                    $registro->doc =  $doc ;
                    $registro->estado = 'CE';
                    $registro->save();
                }
            }

            if ($tipo != 'A' && $datos['total'] == 0) {
                if (substr($datos['doc'],0,3) == 'TIC') {
                    $registro = Tickets::find((int)$resultado);
                    $doc = $this->numeroTicketACero();
                    $registro->doc =  $doc ;
                    $registro->estado = 'CE';
                    $registro->save();
                }
            }

            $prodBP = new ProductosBP();
            foreach($lineas as $linea) {
                unset($linea['id']);
                $linea['ticket_id'] = $resultado ;
                LineasVenta::insert($linea);
                // todoevo actualizar stock
                $resultadoActStock = $prodBP->actStockProducto($linea['producto_id']);
            }
            //$resultadoLineas = LineasVenta::insert($datosLineas);
        }
        // grabamos las lineas

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok','doc'=>$doc);
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }

    public function nuevo(Request $request, Response $response, $args)
    {

        $numTempAlb = 'TIC-'.$this->utiles->generaIdReserva();

        $datos = ['doc'=>$numTempAlb,'fecha'=>date("Y-m-d"),'hora'=>date("H:i") ];
        //todoevo Leer de configuración id cliente y serie por defecto

        try{
            $registro = new Tickets;
            $registro->saveData($datos);
            $resultado = $registro->id;
        } catch (\Exception $e) {
            $resultado = 0 ;
            $mensajes = $e->getMessage() ;
        }
        // leo el albarán y
        $albaran = Tickets::where('id',$registro->id)->get();

        $albaran['totales'] = [];
        $albaran['lineas'] = [];
        $albaran['cobros'] = [];

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok', 'datos'=>$albaran);
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }
    public function leer(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();


        $id = ($datos['id']) ? $datos['id'] : 0;
        $albaran = Tickets::where('id',$id)->first()->toArray();
        $lineasAlbaran = LineasVenta::where('ticket_id',$id)->get()->toArray();

        $albaran['totales'] = [];
        $albaran['lineas'] = $lineasAlbaran;
        $albaran['cobros'] = [];

        $respuesta = array('resultado' => 'ok', 'datos'=>$albaran);

        return $response->withJson($respuesta);
    }

    public function borrarTemporal(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $id = ($datos['id']) ? $datos['id'] : 0;
        // deberíamos de comprobar si el temporal tiene lineas ya qu epuede ser la salida de un abono.
        Tickets::where('id',$id)->delete();
        LineasVenta::where('ticket_id',$id)->delete();

        $respuesta = array('resultado' => 'ok');

        return $response->withJson($respuesta);
    }



    public function numeroTicket() {
        $ticket_id = 0;
        // todoevo hacerlo multiempresa
        try {
            $record = Contadores::where('doc', 'TICKET')->lockForUpdate()->first();
            $ticket_id = $record->valor;
            $record->valor = $ticket_id + 1 ;
            $record->save();
        } catch (\Exception $e) {
            $ticket_id = 0 ;
        }
        $nuevoReg = 'FS-'.str_pad($ticket_id, 6, "0", STR_PAD_LEFT);
        return $nuevoReg ;
    }

    public function numeroTicketACero() {
        $ticket_id = 0;
        // todoevo hacerlo multiempresa
        try {
            $record = Contadores::where('doc', 'TICKETCERO')->lockForUpdate()->first();
            $ticket_id = $record->valor;
            $record->valor = $ticket_id + 1 ;
            $record->save();
        } catch (\Exception $e) {
            $ticket_id = 0 ;
        }
        $nuevoReg = 'TC-'.str_pad($ticket_id, 6, "0", STR_PAD_LEFT);
        return $nuevoReg ;
    }

    public function cobros(Request $request, Response $response, $args)
    {
        $parametros = $request->getParsedBody();


        $id = $parametros['id'];
        $data = DB::table('cobros')
            ->select('cobros.*')
            ->where('cobros.ticket_id','=',$id) ;
        $data->orderBy('cobros.fecha');
        $datos = $data->get();
        $respuesta = array('datos'=>$datos);
        return $response->withJson($respuesta);
    }


    public function imprimir(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();



        $id = ($datos['id']) ? $datos['id'] : 0;
        $albaran = Tickets::where('id',$id)->first()->toArray();
        $lineasAlbaran = LineasVenta::where('ticket_id',$id)->get()->toArray();

        $cobros = Cobros::where('ticket_id',$id)->get()->toArray();

        if ($albaran['cliente_id']) {
            $cliente = Clientes::where('id',$albaran['cliente_id'])->get()->toArray();
            $albaran['cliente'] = $cliente[0];
        }
        else {
            $albaran['cliente'] = array();
        }

        $albaran['totales'] = [];
        $albaran['lineas'] = $lineasAlbaran;
        $albaran['cobros'] = $cobros ;


        $totalFactura = 0;
        $impuestos = array();
        $datosImpuesto = array();

        // Procedimiento para calcular las bases del iva incluido en los albaranes.
        if ($lineasAlbaran) {
            foreach ($lineasAlbaran as $linea) {
                $porcentage = $linea['tipo_iva'];
                if (!isset($impuestos[$porcentage])) {
                    $impuestos[$porcentage] = ['total_iva' => $porcentage, 'base'=> 0,'total'=>0,'tipo_iva'=>$porcentage];
                }
                $impuestos[$porcentage]['total'] = $impuestos[$porcentage]['total'] + $linea['total'];
            };
        }
        foreach ($impuestos as $impuesto) {
            $base = $impuesto['total'] / ( 1 + ($impuesto['tipo_iva'] / 100) ) ;
            $base = round($base,2);
            $impuestos[$porcentage]['total_iva'] = $impuesto['total'] - $base ;
            $impuestos[$porcentage]['base'] = $base;
        };
        $albaran['impuestos'] = $impuestos;

        // renderizamos el html
        $html = $this->app->view->fetch('ticket.twig', ['doc' => $albaran]);

        $respuesta = array('html' => $html);
        return $response->withJson($respuesta);
    }

    public function verProducto(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $producto_id = $datos['id'];
        $fecha = $datos['fecha'];
        $data = DB::table('productos')->where('id', '=', $producto_id)->get();

        $precioTarifa = DB::table('d_tarifas_productos as d')
            ->leftJoin('tarifas_productos as t','t.id','=','d.tarifaproducto_id')
            ->select('d.*')
            ->where('d.producto_id','=',$producto_id)
            ->where('t.fechaini','<=',$fecha)
            ->where('t.fechafin','>=',$fecha)
            ->orderBy('dias','asc')->first();

        $precio = 0 ;
        if (isset($precioTarifa)) {
            $precio = $precioTarifa->precio;
        }

        $array = array('resultado' => 'ok','datos' => $data,'precio'=>$precio,'tarifa'=>$precioTarifa);



        return $response->withJson($array);
    }

    public function listadoDiario(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $cliente_id = ($datos['cliente_id']) ? $datos['cliente_id'] : 0;

        $data = DB::table('lineas_venta as l')
            ->leftJoin('tickets as a','l.ticket_id','=','a.id')
            ->leftJoin('clientes as c','c.id','=','a.cliente_id')
            ->leftJoin('productos as p','p.id','=','l.producto_id')
            ->select('l.*','a.fecha','a.doc','c.razon_social','p.producto','p.familia_desc','p.subfamilia_desc','c.pais')
            ->where('a.fecha','=',$datos['fecha'])
            ->where('a.id','>',0)
            ->get() ;

        $html = $this->view->fetch('listado-tickets-ventas.twig', ['tickets' => $data]);

        $mpdf = new mPDF('c', 'A4', 0, 'Arial', 5, 5, 5, 5, 5, 5);

        $mpdf->WriteHTML($html);
        $nombreFichero = 'ficherostemp3duadjfa/'.'listado-tickets-ventas-productos.pdf';
        $mpdf->Output($nombreFichero, 'F');
        $respuesta = array('link' => $nombreFichero,'datos'=>$data);

        return $response->withJson($respuesta);
    }

    public function listadoCaja(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $fecha = ($datos['fecha']) ? $datos['fecha'] : 0;

        $cobros = DB::table('cobros as c')
            ->leftJoin('clientes', 'clientes.id', '=', 'c.cliente_id')
            ->leftJoin('formaspago as fp', 'fp.id', '=', 'c.formapago_id')
            ->select('c.*','clientes.razon_social','fp.descripcion','fp.tipo')
            ->where('c.fecha', array($fecha))
            ->where('c.ticket_id','>',0)
            ->orderBy('c.formapago')->get();

        $pagosTpv = DB::table('pagostpv as c')
            ->select('c.*')
            ->where('c.fecha', array($fecha))
            ->get();

        $ventas = DB::table('lineas_venta as l')
            ->leftJoin('tickets as a','l.ticket_id','=','a.id')
            ->leftJoin('clientes as c','c.id','=','a.cliente_id')
            ->leftJoin('productos as p','p.id','=','l.producto_id')
            ->select('l.*','a.fecha','a.doc','c.razon_social','p.producto','p.descripcion','c.pais')
            ->where('a.fecha','=',$datos['fecha'])
            ->where('a.id','>',0)
            ->orderBy('l.descripcion','asc')
            ->orderBy('l.producto','asc')
            ->get() ;

        $ventasPro = [];
        foreach($ventas as $v) {
            $ventasPro[] = array('descripcion'=>$v->descripcion,
                'cantidad'=>$v->cantidad,
                'precio'=>$v->precio,
                'total'=>$v->total,
                'producto'=>$v->descripcion.$v->producto);
        }

        $html = $this->view->fetch('listado-caja.twig', ['cobros' => $cobros,'ventas'=>$ventasPro,'pagostpv'=>$pagosTpv,'fecha'=>$fecha]);
        $mpdf = new mPDF('c', 'A4', 0, 'Arial', 5, 5, 5, 5, 5, 5);
        $mpdf->WriteHTML($html);
        $nombreFichero = 'ficherostemp3duadjfa/'.'listado-caja.pdf';
        $mpdf->Output($nombreFichero, 'F');
        $respuesta = array('link' => $nombreFichero,'datos'=>$cobros,'fecha'=>$fecha);


        return $response->withJson($respuesta);
    }


    // todoevo hay que traer los greenfees, no los
    public function nuevoTicketReservaExpress(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $dreservas = $datos['reserva']['dreservas'];
        $reserva = $datos['reserva'];
        $total  = $reserva['total'];

        $reserva_id = $reserva['id'];
        $cliente_id = 1 ; // cliente de contado por defecto
        $nombreReserva = !empty($reserva['nombre_reserva']) ? $datos['nombre_reserva'] : '';
        $referenciaReserva = $reserva['idreserva'];

        // no se factura por dereservas. Se factura por

        $greenfees = $datos['reserva']['greenfees'];

        // hacemos un número temporal por si el
        $numTempTic = 'TIC-'.$this->utiles->generaIdReserva();

        $datos = [
            'doc'=>$numTempTic,
            'fecha'=>date("Y-m-d"),
            'cliente_id' => $cliente_id,
            'reserva_id' => $reserva_id,
            'referencia_reserva' => $referenciaReserva,
            'total' => $total,
            'cobrado' => 0,
            'pendiente'=>$total,
            'estado'=>'CE'
        ];
        // todoevo Leer de configuración id cliente y serie por defecto

        try{
            $registro = new Tickets;
            $registro->saveData($datos);
            $resultado = $registro->id;
        } catch (\Exception $e) {
            $resultado = 0 ;
            $mensajes = $e->getMessage() ;
        }
        // leo el albarán y

        if ($resultado > 0 && $greenfees) {
            // todoevo, traer la referencia de la reserva
            $idTicket = $resultado ;
            // procesamos los greenfees
            // todoevo hacer para los de 9 hoyos
            foreach($greenfees as $gf) {
                $conceptoBuggies = '' ;
             
                $conceptoGF = 'Greenfees' ;
                // linea que generamos
                $datos = [
                    'ticket_id'=>$idTicket,
                    'fecha_reserva'=>$gf['fecha'],
                    'total' => $gf['preciogf'],
                    'descripcion' => 'Greenfees 18 Hoyos',
                    'producto_id' => 0,
                    'producto' => 'GF18', // todoevo, distinguir 9 y 18
                    'tarifa' => $gf['tarifa'],
                    'tarifa_id' => $gf['tarifa_id'],
                    'abono_id' => $gf['abono_id'],
                   // 'abono' => $gf['abono'],
                    'cantidad' => 1,
                    'precio' => $gf['preciogf'],
                    'concepto_reserva'=>$conceptoGF,
                    'referencia_reserva' =>  $gf['idreserva'],
                    'nombre_jugador'=>$gf['nombre_jugador'] ,
                    'hora_reserva'=>$gf['hora'],
                    'recorrido'=> $gf['recorrido'],
                    'es_reserva' => 1,
                    'tipo_iva' => 21
                ];
                $lineaVenta = new LineasVenta();
                $lineaVenta->saveData($datos);

                // todoevo incluir los buggies
                /* No hay problema ya que aquí los cogemos de la dreserva, pero a la hora de factura de otra manera, tenemos que tenerlos distribuidos.


                // si tenemos buggies
                /*if ($gf['buggies'] > 0) {
                    $conceptoBuggies = 'Buggies: ';
                // linea que generamos
                    $datos = [
                        'ticket_id'=>$idTicket,
                        'fecha_reserva'=>$dreserva['fecha'],
                        'total' => $dreserva['total'],
                        'descripcion' => 'Buggie 18 Hoyos',
                        'producto_id' => 0, // todoevo tenemos que buscar en laconfiguración
                      //  'tarifa' => 'tarifa', todoevo
                        'producto' => 'BUGGIE18',
                        'cantidad' => $dreserva['buggies'],
                        'precio' => $dreserva['total'],
                        'concepto_reserva'=>$conceptoBuggies,
                        'referencia_reserva' =>  $dreserva['idreserva'],
                        'nombre_jugador'=>$nombreReserva ,
                        'hora_reserva'=>$dreserva['hora'],
                        'recorrido'=> $dreserva['recorrido'],
                        'es_reserva' => 1,
                        'tipo_iva' => 21
                    ];
                    $lineaVenta = new LineasVenta();
                    $lineaVenta->saveData($datos);
                }*/
            }
        }

        $ticket = Tickets::where('id',$registro->id)->first()->toArray();

        foreach($greenfees as $gf) {
            Greenfees::where('idreservahora', $gf['idreservahora'])->update(['ticket_id' => $idTicket]);
        }

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok', 'datos'=>$ticket);
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }


    public function facturarGreenfees(Request $request, Response $response, $args)
    {
        $datos = $request->getParsedBody();

        $reserva = $datos['reserva'];
        // $dreservas = $datos['reserva']['dreservas'];
        $greenfees = $datos['reserva']['greenfees'];

        $total  = $reserva['total'];

        $reserva_id = $reserva['id'];
        $cliente_id = 1 ; // cliente de contado por defecto
        $referenciaReserva = $reserva['idreserva'];

        // no se factura por dereservas. Se factura por greenfees. No los vamos a agrupar en principio.
        // hacemos un número temporal por si el
        $numTempTic = 'TIC-'.$this->utiles->generaIdReserva();

        $datos = [
            'doc'=>$numTempTic,
            'fecha'=>date("Y-m-d"),
            'cliente_id' => $cliente_id,
            'reserva_id' => $reserva_id,
            'referencia_reserva' => $referenciaReserva,
            'total' => $total,
            'cobrado' => 0,
            'pendiente'=>$total,
            'estado'=>'CE'
        ];
        // todoevo Leer de configuración id cliente y serie por defecto

        try{
            $registro = new Tickets;
            $registro->saveData($datos);
            $resultado = $registro->id;
        } catch (\Exception $e) {
            $resultado = 0 ;
            $mensajes = $e->getMessage() ;
        }
        // leo el albarán y

        if ($resultado > 0 && $greenfees) {
            // todoevo, traer la referencia de la reserva
            $idTicket = $resultado ;
            // procesamos los greenfees
            // todoevo hacer para los de 9 hoyos
            foreach($greenfees as $gf) {
                $concepto = 'Greenfees' ;

                $datos = [
                    'ticket_id'=>$idTicket,
                    'fecha_reserva'=>$gf['fecha'],
                    'total' => $gf['preciogf'],
                    'descripcion' => 'Greenfees 18 Hoyos',
                    'producto_id' => 0,
                    'producto' => 'GF18', // todoevo, distinguir 9 y 18
                    'tarifa' => $gf['tarifa'],
                    'tarifa_id' => $gf['tarifa_id'],
                    'abono_id' => $gf['abono_id'],
                    // 'abono' => $gf['abono'],
                    'cantidad' => 1,
                    'precio' => $gf['preciogf'],
                    'concepto_reserva'=>$concepto,
                    'referencia_reserva' =>  $gf['idreserva'],
                    'nombre_jugador'=>$gf['nombre_jugador'] ,
                    'hora_reserva'=>$gf['hora'],
                    'recorrido'=> $gf['recorrido'],
                    'es_reserva' => 1,
                    'tipo_iva' => 21
                ];
                $lineaVenta = new LineasVenta();
                $lineaVenta->saveData($datos);

                // todoevo incluir los buggies
                /* No hay problema ya que aquí los cogemos de la dreserva, pero a la hora de factura de otra manera, tenemos que tenerlos distribuidos.

                // si tenemos buggies
                /*if ($gf['buggies'] > 0) {
                    $conceptoBuggies = 'Buggies: ';
                // linea que generamos
                    $datos = [
                        'ticket_id'=>$idTicket,
                        'fecha_reserva'=>$dreserva['fecha'],
                        'total' => $dreserva['total'],
                        'descripcion' => 'Buggie 18 Hoyos',
                        'producto_id' => 0, // todoevo tenemos que buscar en laconfiguración
                      //  'tarifa' => 'tarifa', todoevo
                        'producto' => 'BUGGIE18',
                        'cantidad' => $dreserva['buggies'],
                        'precio' => $dreserva['total'],
                        'concepto_reserva'=>$conceptoBuggies,
                        'referencia_reserva' =>  $dreserva['idreserva'],
                        'nombre_jugador'=>$nombreReserva ,
                        'hora_reserva'=>$dreserva['hora'],
                        'recorrido'=> $dreserva['recorrido'],
                        'es_reserva' => 1,
                        'tipo_iva' => 21
                    ];
                    $lineaVenta = new LineasVenta();
                    $lineaVenta->saveData($datos);
                }*/
            }
        }

        $ticket = Tickets::where('id',$registro->id)->first()->toArray();

        // Actualizamos los greenfees
        foreach($greenfees as $gf) {
            Greenfees::where('id', $gf['id'])->update(['ticket_id' => $idTicket]);
        }

        if ($resultado > 0) {
            $respuesta = array('resultado' => 'ok', 'datos'=>$ticket);
        } else {
            $respuesta = array('resultado' => 'ko','mensajes'=>$mensajes);
        }

        return $response->withJson($respuesta);
    }

}